# This file is part of pyDC software
# Copyright 2002-2004 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from EventGenerator import *
from JobViewer import *
from JobViewerInfo import *
from MultirowMessage import *
from time import strftime
from Util import getMainWnd
import wx

from DCHub import *
import DCWorker

class ChatJob(EventGenerator):
	def __init__(self, nick, hub):
		EventGenerator.__init__(self)
		
		self.nick = nick
		self.hub = hub
		self.address = hub.getAddress()
		self.rows = []
		
		self.sentAwayMessage = False
	
	def stop(self):
		pass

	def appendMsg(self, msg, received = True):
		self.rows.append(msg.replace('\r', ''))
		
		self.lock.acquire()
		for listener in self.getListeners():
			listener.onMsg(self.nick, msg, received)
		self.lock.release()
		
		if received and getMainWnd().isAway() and not self.sentAwayMessage:
			self.sentAwayMessage = True
			self.sendMsg(DCWorker.getWorker().getSettings().getCustom('awayMessage'))

	def sendMsg(self, msg):
		if self.address != self.hub.getAddress():
			#TODO: alert user
			return
		
		self.hub.sendMsg(msg, self.nick)
		self.appendMsg('<%s> %s\n' % (self.hub.getNick(), msg), False)

	def getNick(self):
		return self.nick

	def getAddress(self):
		return self.address

	def getRows(self):
		return self.rows
	
	def onAway(self):
		self.sentAwayMessage = False

class ChatJobEventListener:
	def onMsg(self, nick, msg, received):
		pass

class ChatViewer(JobViewer, ChatJobEventListener):
	def __init__(self, parent, id, info):
		JobViewer.__init__(self, parent, id, info)
		self.custom = DCWorker.getWorker().getSettings().getCustom
		
		self.text = wx.TextCtrl(self, -1, style = wx.TE_MULTILINE | wx.TE_RICH)
		self.msg = wx.TextCtrl(self, 1, style = wx.TE_PROCESS_ENTER)
		multirowMsg = wx.Button(self, 3, 'M', style = wx.BU_EXACTFIT)
		close = wx.Button(self, 2, 'Close')

		hLayout = wx.BoxSizer(wx.HORIZONTAL)
		hLayout.Add(self.msg, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 2)
		hLayout.Add(multirowMsg, 0, wx.EXPAND | wx.RIGHT, 2)

		vLayout = wx.BoxSizer(wx.VERTICAL)
		vLayout.Add(self.text, 1, wx.EXPAND)
		vLayout.Add(hLayout, 0, wx.EXPAND | wx.BOTTOM | wx.TOP, 5)
		vLayout.Add(close, 0, wx.ALIGN_CENTER | wx.BOTTOM | wx.TOP, 5)

		self.SetSizer(vLayout)
		self.Layout()

		wx.EVT_TEXT_ENTER(self, 1, self.onSend)
		wx.EVT_BUTTON(self, 2, self.onClose)
		wx.EVT_BUTTON(self, 3, self.onMultirowMsg)

		self.text.SetEditable(False)
	
	def getTitle(self):
		return self.info.getJob().getNick()

	def onMsg(self, nick, msg, received):
		#NOTE: this implementation assumes that this
		#      function is executed by main (GUI) thread
		if not received:
			self.appendRow(msg, wx.Color(0,128,128))
		else:
			self.appendRow(msg)

	def onSend(self, event):
		self.sendMsg(self.msg.GetValue())
		self.msg.SetValue('')

	def onClose(self, event):
		self.viewerDestroy(True)
		
	def onMultirowMsg(self, event):
		mm = MultirowMessage(self, self.info.getJob().getNick())
		mm.Center()
		if mm.ShowModal() == wx.ID_OK:
			self.sendMsg(mm.GetValue())

	#=================#
	# Private section #
	#=================#
	
	def sendMsg(self, text):
		if len(text) == 0: return
		
		try:
			self.info.getJob().sendMsg(text)
		except ValueError, e:
			self.appendRow(e[0] + '\n', 'RED')
		
		wnd = getMainWnd()
		if wnd.isAway():
			wnd.setAway(False)
			self.appendRow('Away mode automatically disabled.\n', 'RED')

	def appendRow(self, row, color = None):
		if self.custom('showTimestamps'):
			row = '[%s] %s' % (strftime('%H:%M:%S'), row)
			
		if color: self.text.SetDefaultStyle(wx.TextAttr(color))
		self.text.AppendText(row)
		if color: self.text.SetDefaultStyle(wx.TextAttr('BLACK'))

		lastPos = max(self.text.GetLastPosition() - 1, 0)
		self.text.ShowPosition(lastPos)

		if self.text.GetNumberOfLines() > DCHub.MAX_LOG_ROWS:
			self.text.Remove(0, self.text.GetLineLength(0))

class ChatViewerInfo(JobViewerInfo): pass
