# This file is part of pyDC software
# Copyright 2004 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from Util import showErrorMessage

import wx
import DCWorker

class FavoriteDialog(wx.Dialog):
	def __init__(self, parent, favorite):
		wx.Dialog.__init__(self, parent, -1, 'Favorite', style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
		self.oldaddr = favorite[0]
		self.favorite = favorite

		# Network box
		network = wx.StaticBox(self, -1, 'Network')
		self.address = wx.TextCtrl(self, -1)
		self.autoconnect = wx.Choice(self, -1, choices = [ 'no', 'yes' ])

		gs1 = wx.FlexGridSizer(cols = 2, hgap = 5, vgap = 5)
		gs1.AddGrowableCol(1)
		gs1.Add(wx.StaticText(self, -1, 'Address:'), 0, wx.ALIGN_CENTER_VERTICAL)
		gs1.Add(self.address, 1, wx.EXPAND)
		gs1.Add(wx.StaticText(self, -1, 'Autoconnect:'), 0, wx.ALIGN_CENTER_VERTICAL)
		gs1.Add(self.autoconnect, 1, wx.EXPAND)
 
		bs1 = wx.StaticBoxSizer(network, wx.VERTICAL)
		bs1.Add(gs1, 1, wx.EXPAND | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)
		# END network box
		
		# Authentication box
		auth = wx.StaticBox(self, -1, 'Authentication')
		self.nick = wx.TextCtrl(self, -1)
		self.password = wx.TextCtrl(self, -1, style = wx.TE_PASSWORD)
		
		gs2 = wx.FlexGridSizer(cols = 2, hgap = 5, vgap = 5)
		gs2.AddGrowableCol(1)
		gs2.Add(wx.StaticText(self, -1, 'Nick:'), 0, wx.ALIGN_CENTER_VERTICAL)
		gs2.Add(self.nick, 1, wx.EXPAND)
		gs2.Add(wx.StaticText(self, -1, 'Password:'), 0, wx.ALIGN_CENTER_VERTICAL)
		gs2.Add(self.password, 1, wx.EXPAND)
		
		bs2 = wx.StaticBoxSizer(auth, wx.VERTICAL)
		bs2.Add(gs2, 1, wx.EXPAND | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)
		# END authentication box
		
		# Button panel
		bs3 = wx.BoxSizer(wx.HORIZONTAL)
		bs3.Add(wx.Button(self, wx.ID_OK, 'OK'), 0, wx.RIGHT, 10)
		bs3.Add(wx.Button(self, wx.ID_CANCEL, 'Cancel'))

		# Final layout setup
		bs4 = wx.BoxSizer(wx.VERTICAL)
		bs4.Add(bs1, 1, wx.EXPAND | wx.ALL, 5)
		bs4.Add(bs2, 1, wx.EXPAND | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)
		bs4.Add(bs3, 0, wx.ALIGN_RIGHT | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)
		self.SetSizer(bs4)
		
		bs4.Fit(self)
		width, height = self.GetSizeTuple()
		self.SetSize(wx.Size(max(300, width), height))
		
		# Fill fields
		self.address.SetValue(self.favorite[0])
		self.autoconnect.SetSelection(self.favorite[1])
		self.nick.SetValue(self.favorite[2] or '')
		self.password.SetValue(self.favorite[3] or '')

		# Setup events
		wx.EVT_BUTTON(self, wx.ID_OK, self.onOK)

	def onOK(self, event):
		try:
			address = self.address.GetValue()
			autoconnect = self.autoconnect.GetStringSelection() == 'yes'
			nick = self.nick.GetValue()
			password = self.password.GetValue()
			 
			DCWorker.getWorker().getSettings().updateFavorite(self.oldaddr, address, autoconnect, nick, password)
			self.EndModal(wx.ID_OK)
		
		except ValueError:
			showErrorMessage(self, 'You have entered an invalid value.')
