# This file is part of pyDC software
# Copyright 2002-2004 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from copy import copy
from JobViewer import *
from JobViewerInfo import *
from MultirowMessage import *
from thread import start_new_thread
from time import localtime, strftime, time
from threading import Lock
from Util import *
import wx

from DCHub import *
from DCHubEventListener import *
from DCUser import DCUser
from DCUserList import *
import DCWorker

class HubViewer(JobViewer, DCHubEventListener):
	def __init__(self, parent, id, info):
		self.dusers = []
	
		self.usern = {}
		self.useri = {}
		self.userr = []
		self.pendingUpdate = False
		JobViewer.__init__(self, parent, id, info)
		
		self.users = []
		self.updateStamp = 0
		self.menuId = -1
		self.firstRow = True
		self.sortAlgos = [ (None, True), (caseFilter, False), (None, False), (None, False), (None, True), (None, True) ]

		# Controls & layout
		self.notebook = wx.Notebook(self, -1)

		chatPanel = wx.Panel(self.notebook, -1)
		self.text = wx.TextCtrl(chatPanel, -1, style = wx.TE_MULTILINE | wx.TE_READONLY |  wx.TE_RICH2 | wx.TE_WORDWRAP)
		self.msg = wx.TextCtrl(chatPanel, 1, style = wx.TE_PROCESS_ENTER)
		multirowMsg = wx.Button(chatPanel, 6, 'M', style = wx.BU_EXACTFIT)

		chLayout = wx.BoxSizer(wx.HORIZONTAL)
		chLayout.Add(self.msg, 2, wx.EXPAND | wx.LEFT | wx.RIGHT, 2)
		chLayout.Add(multirowMsg, 0, wx.EXPAND | wx.RIGHT, 2)

		cvLayout = wx.BoxSizer(wx.VERTICAL)
		cvLayout.Add(self.text, 1, wx.EXPAND)
		cvLayout.Add(chLayout, 0, wx.EXPAND | wx.BOTTOM | wx.TOP, 5)
		chatPanel.SetSizer(cvLayout)

		self.notebook.AddPage(chatPanel, 'Chat')

		usersPanel = wx.Panel(self.notebook, -1)
		self.list = wx.ListCtrl(usersPanel, 2, style = wx.LC_REPORT | wx.LC_VRULES | wx.LC_SINGLE_SEL)
		self.list.InsertColumn(0, 'OP')
		self.list.InsertColumn(1, 'Nick')
		self.list.InsertColumn(2, 'Description')
		self.list.InsertColumn(3, 'Email')
		self.list.InsertColumn(4, 'Connection')
		self.list.InsertColumn(5, 'Share', wx.LIST_FORMAT_RIGHT)
		
		self.notebook.AddPage(usersPanel, 'Users')
		
		# Layout
		uvLayout = wx.BoxSizer(wx.VERTICAL)
		uvLayout.Add(self.list, 1, wx.EXPAND)
		usersPanel.SetSizer(uvLayout)
		
		self.list.Freeze()
		self.list.InsertStringItem(0, '0')
		self.list.SetStringItem(0, 3, 'EMAIL')
		self.list.SetStringItem(0, 4, 'MODEM')
		self.list.SetStringItem(0, 5, '999.99 GB')
		self.list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
		self.list.SetColumnWidth(3, wx.LIST_AUTOSIZE)
		self.list.SetColumnWidth(4, wx.LIST_AUTOSIZE)
		self.list.SetColumnWidth(5, wx.LIST_AUTOSIZE)
		self.list.DeleteItem(0)
		self.list.Thaw()

		bhLayout = wx.BoxSizer(wx.HORIZONTAL)
		bhLayout.Add(wx.Button(self, 3, 'Dismiss'), 0, wx.RIGHT, 10)
		bhLayout.Add(wx.Button(self, 4, 'Disconnect'), 0, wx.RIGHT, 10)
		bhLayout.Add(wx.Button(self, 5, 'Reconnect'))

		vLayout = wx.BoxSizer(wx.VERTICAL)
		vLayout.Add(self.notebook, 1, wx.EXPAND)
		vLayout.Add(bhLayout, 0, wx.ALIGN_CENTER | wx.BOTTOM | wx.TOP, 5)

		self.SetSizer(vLayout)
		self.Layout()

		# Events
		wx.EVT_TEXT_ENTER(self, 1, self.onMsg)
		wx.EVT_LIST_COL_CLICK(self, 2, self.setSortAlgo)
		wx.EVT_SIZE(self.list, self.onListResize)
		wx.EVT_LIST_ITEM_RIGHT_CLICK(self, 2, self.onUserOptions)
		wx.EVT_LIST_ITEM_ACTIVATED(self, 2, self.onUserSelected)
		wx.EVT_MENU_RANGE(self.list, 500, 504, self.onPopupSelection)
		wx.EVT_BUTTON(self, 3, self.onDismiss)
		wx.EVT_BUTTON(self, 4, self.onDisconnect)
		wx.EVT_BUTTON(self, 5, self.onReconnect)
		wx.EVT_BUTTON(self, 6, self.onMultirowMsg)
		self.Connect(1, 1, wx.wxEVT_NULL, self.updateInfo)
		self.Connect(2, 2, wx.wxEVT_NULL, self.addLogRow)
		self.Connect(3, 3, wx.wxEVT_NULL, self.update)
		self.Connect(4, 4, wx.wxEVT_NULL, self.fullUpdate)
		self.Connect(5, 5, wx.wxEVT_NULL, self.clearAll)
		
		# Final setup
		self.notebook.SetSelection(self.info.page)
		
		self.text.SetEditable(False)
		rows = self.info.getJob().getLogRows()
		if len(rows):
			self.text.Freeze()
			for i in rows[:-1]: self.appendText(i)
			self.text.Thaw()

			self.text.ShowPosition(self.text.GetLastPosition())
			self.AddPendingEvent(guiEvent(2, rows[-1]))
		
		if self.info.msgText != None:
			self.msg.SetValue(self.info.msgText)

		for col in range(len(self.info.columnWidth)):
			self.list.SetColumnWidth(col, self.info.columnWidth[col])

		self.users = self.usern.values()
		self.usern = {}
		if len(self.users) > 0:
			self.fullUpdate()
	
	def onMsg(self, event):
		text = self.msg.GetValue()
		if len(text) == 0: return
		w = DCWorker.getWorker()

		if text[0] == '/':
			text = text[1:]
			if text[:4].lower() == 'away':
				msg = text[5:]
				if len(msg): w.setCustom('awayMessage', msg)
				getMainWnd.setAway(True)
				self.AddPendingEvent(guiEvent(2, (DCHub.LOG_STATUS, localtime(), 'Entering away mode.\n')))
			
			elif text.lower() == 'back':
				getMainWnd().setAway(False)
				self.AddPendingEvent(guiEvent(2, (DCHub.LOG_STATUS, localtime(), 'Leaving away mode.\n')))
				
			elif text.lower() == 'refresh':
				w.refreshLocalList()
				
			elif text.lower() == 'fav':
				try:
					getMainWnd().addFavorite('%s:%d' % self.info.getJob().getAddress())
				except ValueError:
					self.AddPendingEvent(guiEvent(2, (DCHub.LOG_ERROR, localtime(), 'Invalid hub address.\n')))
			
			elif text.lower() == 'time':
				show = not w.getCustom('showTimestamps')
				w.setCustom('showTimestamps', show)
				
				if show: status = 'on'
				else: status = 'off'
				self.AddPendingEvent(guiEvent(2, (DCHub.LOG_STATUS, localtime(), 'Timestamps: %s.\n' % status)))
			
			else:
				self.AddPendingEvent(guiEvent(2, (DCHub.LOG_ERROR, localtime(), 'Unknown command \'%s\'.\n' % text)))
		
		else:
			self.sendMsg(text)
		
		self.msg.SetValue('')

	def setSortAlgo(self, event):
		col = event.GetColumn()
		if self.info.sortColumn == col:
			self.info.sortReverse = not self.info.sortReverse
		else:
			self.info.sortColumn = col
		self.fullUpdate()
	
	def onListResize(self, event):
		colWidths = [ self.list.GetColumnWidth(col) for col in range(6) ]
		width = getListWidth(self.list) - colWidths[0] - colWidths[3] - colWidths[4] - colWidths[5]
		if width > 0:
			oldWidth = colWidths[1] + colWidths[2]
			for i in range(1, 3):
				self.list.SetColumnWidth(i, int(float(colWidths[i]) / oldWidth * width))
		event.Skip()

	def onUserOptions(self, event):
		try: nick = self.users[event.GetIndex()].getNick()
		except IndexError: return

		hub = self.info.getJob()
		if hub.getNick() == nick: return

		self.list.SetItemState(event.GetIndex(), wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

		menu = wx.Menu()
		menu.Append(500, 'Chat')
		menu.Append(501, 'Get file list')
		menu.Append(502, 'Copy nick')
		menu.Append(503, 'Remove user from queue')
		if hub.isOp():
			menu.Append(504, 'Kick')

		self.list.PopupMenu(menu, event.GetPoint())

		if self.menuId == 500:
			getMainWnd().getChat(nick, self.info.getJob())
			
		elif self.menuId == 501:
			try:
				u = hub.getUserByNick(nick)
				if u == None: raise ValueError
				l = DCUserList(u)
			except ValueError:
				showErrorMessage(self, 'No such user.')
		
		elif self.menuId == 502:
			clipdata = wx.TextDataObject() 
			clipdata.SetText(nick)
			wx.TheClipboard.Open() 
			wx.TheClipboard.SetData(clipdata) 
			wx.TheClipboard.Close()
		
		elif self.menuId == 503:
			for item in DCWorker.getWorker().getQueue().getItems():
				item.removeSource(nick)
		
		elif self.menuId == 504:
			self.info.getJob().kick(nick)
		
		self.menuId = -1
	
	def onUserSelected(self, event):
		try:
			nick = self.users[event.GetIndex()].getNick()
			getMainWnd().getChat(nick, self.info.getJob())
		except IndexError: pass
	
	def onPopupSelection(self, event):
		self.menuId = event.GetId()
	
	def onDismiss(self, event):
		self.info.page = self.notebook.GetSelection()
		if self.info.page == -1: self.info.page = 0
		
		self.info.msgText = self.msg.GetValue()
		
		self.info.columnWidth = []
		for col in range(self.list.GetColumnCount()):
			self.info.columnWidth.append(self.list.GetColumnWidth(col))
		
		self.viewerDestroy(False)

	def onDisconnect(self, event):
		self.viewerDestroy(True)
		
	def onReconnect(self, event):
		job = self.info.getJob()
		alive = job.isAlive()
		job.reconnect(True)
		if not alive: DCWorker.getWorker().addJob(job)
		
	def onMultirowMsg(self, event):
		mm = MultirowMessage(self)
		mm.Center()
		if mm.ShowModal() == wx.ID_OK:
			msg = mm.GetValue()
			if len(msg) == 0: return
			self.sendMsg(msg)

	def onHubInfo(self, hub):
		self.AddPendingEvent(wx.PyEvent(1))

	def onHubDisconnection(self, hub):
		self.AddPendingEvent(wx.PyEvent(5))
		
	def onNewUsers(self, hub, users):
		self.lock.acquire()
		for user in users:
			self.usern[user.nick] = user
		self.lock.release()

	def onUserDisconnections(self, hub, users):
		self.lock.acquire()
		for user in users:
			if self.useri.has_key(user.nick):
				del self.useri[user.nick]
		
			if self.usern.has_key(user.nick):
				del self.usern[user.nick]
			else:
				self.userr.append(user)
		self.lock.release()

	def onUserInfos(self, hub, users):
		self.lock.acquire()
		for user in [ u for u in users if not self.usern.has_key(u.nick) ]:
			self.useri[user.nick] = user
		self.lock.release()

	def onLogUpdate(self, hub, row):
		self.AddPendingEvent(guiEvent(2, row))
	
	def getTitle(self):
		return self.info.getJob().getName()

	def updateInfo(self, event):
		title = self.getTitle()
		self.lock.acquire()
		for listener in self.getListeners():
			listener.onViewerTitle(self, title)
		self.lock.release()
	
	def addLogRow(self, event):
		self.appendText(event.data)
		if self.text.GetNumberOfLines() == DCHub.MAX_LOG_ROWS:
			self.text.Remove(0, self.text.GetLineLength(0))
		
		if wx.Platform == '__WXMSW__':
			self.text.PageDown()
		else:
			self.text.ShowPosition(self.text.GetLastPosition())
	
	def fullUpdate(self, event = None):
		if self.pendingUpdate:
			return self.AddPendingEvent(wx.PyEvent(4))
		
		snap = self.snap()
		filter, reverse = self.sortAlgos[self.info.sortColumn]
		if self.info.sortReverse: reverse = not reverse
		if filter != None:
			aux = [ (filter(s[self.info.sortColumn]), s) for s in snap ]
		else:
			aux = [ (s[self.info.sortColumn], s) for s in snap ]

		aux.sort()
		if reverse: aux.reverse()

		self.list.Freeze()
		self.list.DeleteAllItems()
		for pos, s in enumerate([ a[1] for a in aux ]):
			self.users[pos] = s[-1]
			self.list.InsertStringItem(pos, str(s[0]))
			self.updateUser(pos, s, 2)
		self.list.Thaw()
	
	def clearAll(self, event):
		self.lock.acquire()
		self.usern = {}
		self.useri = {}
		self.userr = []
		self.lock.release()
		self.users = []

		self.list.DeleteAllItems()

	def update(self, event = None):
		if event == None:
			if self.pendingUpdate: return
		
			self.lock.acquire()
			un, ui, ur = self.usern, self.useri, self.userr
			self.usern, self.useri, self.userr = {}, {}, []
			self.lock.release()
			
			self.pendingUpdate = True
			start_new_thread(self.updateWorker, (self.info.sortColumn, self.info.sortReverse, copy(self.users), un, ui, ur))
		else:
			self.pendingUpdate = False
			self.users = event.data[0]
			
			self.list.Freeze()
			for pos in event.data[3]:
				self.list.DeleteItem(pos)
			for pos, snap in event.data[2]:
				self.updateUser(pos, snap, 3)
			for pos, snap in event.data[1]:
				self.list.InsertStringItem(pos, str(snap[0]))
				self.updateUser(pos, snap, 2)
			self.list.Thaw()

	#=================#
	# Private section #
	#=================#

	def snap(self, users = None):
		if users == None: users = self.users
		return [ (u.isOp(), u.getNick(), u.getDescription(), u.getEmail(), u.getConnection(), u.getShare(), u) for u in users ]
		
	def sendMsg(self, msg):
		try:
			self.info.getJob().sendMsg(msg)
		except ValueError, e:
			self.AddPendingEvent(guiEvent(2, (DCHub.LOG_ERROR, localtime(), e[0])))

	def appendText(self, row):
		msg = row[2][:-1].replace('\r', '')
		
		if DCWorker.getWorker().getSettings().getCustom('showTimestamps'):
			msg = '[%s] %s' % (strftime('%H:%M:%S', row[1]), msg)
		
		if self.firstRow:
			self.firstRow = False
		else:
			msg = '\n' + msg

		if row[0] == DCHub.LOG_CHAT:
			self.text.AppendText(msg)
		
		elif row[0] == DCHub.LOG_STATUS:
			self.text.SetDefaultStyle(wx.TextAttr(wx.Color(0,128,128)))
			self.text.AppendText(msg)
			self.text.SetDefaultStyle(wx.TextAttr('BLACK'))
		
		elif row[0] == DCHub.LOG_PRIVCHAT:
			#self.text.AppendText('Priv: ' + msg)
			pass

		elif row[0] == DCHub.LOG_ERROR:
			self.text.SetDefaultStyle(wx.TextAttr('RED'))
			self.text.AppendText(msg)
			self.text.SetDefaultStyle(wx.TextAttr('BLACK'))
			
	def updateWorker(self, sortColumn, sortReverse, users, un, ui, ur):
		try:
			nlog, ilog, rlog = [], [], []
			
			for user in ur:
				idx = users.index(user)
				rlog.append(idx)
				del users[idx]
			
			snap = self.snap(users)
			filter, reverse = self.sortAlgos[sortColumn]
			if sortReverse: reverse = not reverse
			if filter != None:
				aux = [ (filter(s[sortColumn]), s) for s in snap ]
			else:
				aux = [ (s[sortColumn], s) for s in snap ]
			
			aux.sort()
			if reverse: aux.reverse()
			
			for idx, s in enumerate([ a[1] for a in aux ]):
				if users[idx] != s[-1]:
					ilog.append((idx, s))
				elif ui.has_key(s[1]):
					ilog.append((idx, s))
			
			if filter != None:
				aux += [ (filter(s[sortColumn]), s) for s in self.snap(un.itervalues()) ]
			else:
				aux += [ (s[sortColumn], s) for s in self.snap(un.itervalues()) ]
			
			aux.sort()
			if reverse: aux.reverse()
			users = [ a[1][-1] for a in aux ]
			
			if len(un) > len(users) / 2:
				for idx, user in enumerate(users):
					if un.has_key(user.nick):
						nlog.append((idx, aux[idx][1]))
			else:
				for user in un.itervalues():
					idx = users.index(user)
					nlog.append((idx, aux[idx][1]))
			nlog.sort()

			self.AddPendingEvent(guiEvent(3, (users, nlog, ilog, rlog)))
		except wx.PyDeadObjectError:
			pass

	def updateUser(self, pos, snap, mask):
		if mask & 1:
			self.list.SetStringItem(pos, 0, str(snap[0]))

		if mask & 2:
			self.list.SetStringItem(pos, 1, snap[1])

			if snap[2] == None:
				self.list.SetStringItem(pos, 2, '')
			else:
				self.list.SetStringItem(pos, 2, snap[2])

			if snap[3] == None:
				self.list.SetStringItem(pos, 3, '')
			else:
				self.list.SetStringItem(pos, 3, snap[3])

			self.list.SetStringItem(pos, 4, DCUser.CONN_NAMES[snap[4]])

			if snap[5] == -1:
				self.list.SetStringItem(pos, 5, '')
			else:
				self.list.SetStringItem(pos, 5, formatSize(snap[5]))

class HubViewerInfo(JobViewerInfo):
	def __init__(self, hub):
		JobViewerInfo.__init__(self, hub)

		self.page = 0
		self.msgText = None
		self.sortColumn = 1
		self.sortReverse = False
		self.columnWidth = []
