# This file is part of pyDC software
# Copyright 2002-2003 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from Favorite import FavoriteDialog
from Util import getMainWnd, getListWidth, showErrorMessage
import sys
import wx

from DCHublistRetriever import *
from DCHublistRetrieverEventListener import *
import DCWorker

class HublistPanel(wx.Panel, DCHublistRetrieverEventListener):
	def __init__(self, parent, id):
		wx.Panel.__init__(self, parent, id)

		self.sortAlgos = [ (self.itemFilter, False), (None, True), (self.itemFilter, False), (None, True) ]
		self.sortColumn = 3
		self.sortReverse = False
		self.hubList = []
		self.vetoing = False

		# Controls
		notebook = wx.Notebook(self, -1, style = wx.NB_BOTTOM)
		
		listPanel = wx.Panel(notebook, -1)
		refresh = wx.Button(listPanel, 1, 'Refresh')
		self.list = wx.ListCtrl(listPanel, 2, style = wx.LC_REPORT | wx.LC_VRULES)
		self.addr = wx.TextCtrl(listPanel, 3, style = wx.TE_PROCESS_ENTER)
		connect = wx.Button(listPanel, 4, 'Connect')

		self.list.InsertColumn(0, 'Name')
		self.list.InsertColumn(1, 'Address')
		self.list.InsertColumn(2, 'Description')
		self.list.InsertColumn(3, 'Users', wx.LIST_FORMAT_RIGHT)
		
		favPanel = wx.Panel(notebook, -1)
		self.favlist = wx.ListCtrl(favPanel, 5, style = wx.LC_REPORT | wx.LC_VRULES | wx.LC_EDIT_LABELS)
		self.favaddr = wx.TextCtrl(favPanel, 6, style = wx.TE_PROCESS_ENTER)
		add = wx.Button(favPanel, 7, 'Add')
		remove = wx.Button(favPanel, 8, 'Remove')
		edit = wx.Button(favPanel, 9, 'Edit')
		
		self.favlist.InsertColumn(0, 'Address')
		self.favlist.InsertColumn(1, 'Autoconnect')
		
		# Layout
		lvLayout = wx.BoxSizer(wx.VERTICAL)
		lhLayout = wx.BoxSizer(wx.HORIZONTAL)

		lhLayout.Add(self.addr, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 2)
		lhLayout.Add(connect, 0, wx.LEFT | wx.RIGHT, 2)
		lvLayout.Add(refresh, 0, wx.EXPAND | wx.TOP, 5)
		lvLayout.Add(self.list, 1, wx.EXPAND | wx.TOP, 5)
		lvLayout.Add(lhLayout, 0, wx.EXPAND | wx.TOP | wx.BOTTOM, 3)
		listPanel.SetSizer(lvLayout)
		
		fvLayout = wx.BoxSizer(wx.VERTICAL)
		fhLayout = wx.BoxSizer(wx.HORIZONTAL)
		
		fhLayout.Add(add, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 2)
		fhLayout.Add(remove, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 2)
		fhLayout.Add(edit, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 2)
		fvLayout.Add(self.favlist, 1, wx.EXPAND | wx.TOP | wx.BOTTOM, 5)
		fvLayout.Add(self.favaddr, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 2)
		fvLayout.Add(fhLayout, 0, wx.EXPAND | wx.TOP | wx.BOTTOM, 3)
		favPanel.SetSizer(fvLayout)
		
		notebook.AddPage(listPanel, 'Full list')
		notebook.AddPage(favPanel, 'Favorites')
		
		self.list.Freeze()
		self.list.InsertStringItem(0, '###')
		self.list.SetStringItem(0, 3, '9999')
		self.list.SetColumnWidth(3, wx.LIST_AUTOSIZE)
		self.list.DeleteItem(0)
		self.list.Thaw()
		
		self.favlist.SetColumnWidth(1, 32)
		
		vLayout = wx.BoxSizer(wx.VERTICAL)
		vLayout.Add(notebook, 1, wx.EXPAND)
		self.SetSizer(vLayout)
		self.Layout()
		
		# Event handlers
		wx.EVT_BUTTON(self, 1, self.onRefresh)
		wx.EVT_LIST_COL_CLICK(self, 2, self.setSortAlgo)
		wx.EVT_SIZE(self.list, self.onListResize)
		wx.EVT_LIST_ITEM_ACTIVATED(self, 2, self.onListConnect)
		wx.EVT_LIST_ITEM_RIGHT_CLICK(self, 2, self.onListOptions)
		wx.EVT_MENU_RANGE(self.list, 400, 401, self.onListPopup)
		wx.EVT_TEXT_ENTER(self, 3, self.onManualConnect)
		wx.EVT_BUTTON(self, 4, self.onManualConnect)
		wx.EVT_SIZE(self.favlist, self.onFavlistResize)
		wx.EVT_LIST_ITEM_SELECTED(self, 5, self.onFavlistSelection)
		wx.EVT_LIST_END_LABEL_EDIT(self, 5, self.onFavlistEdit)
		wx.EVT_LIST_ITEM_ACTIVATED(self, 5, self.onFavlistConnect)
		wx.EVT_LIST_ITEM_RIGHT_CLICK(self, 5, self.onFavlistOptions)
		wx.EVT_MENU_RANGE(self.favlist, 500, 502, self.onFavlistPopup)
		wx.EVT_TEXT_ENTER(self, 6, self.onFavAdd)
		wx.EVT_BUTTON(self, 7, self.onFavAdd)
		wx.EVT_BUTTON(self, 8, self.onFavRemove)
		wx.EVT_BUTTON(self, 9, self.onFavEdit)
		self.Connect(1, 1, wx.wxEVT_NULL, self.update)
		self.Connect(2, 2, wx.wxEVT_NULL, self.displayError)

	def onRefresh(self, event):
		retriever = DCHublistRetriever()
		retriever.registerListener(self)
		DCWorker.getWorker().addJob(retriever)

	def setSortAlgo(self, event):
		col = event.GetColumn()
		if self.sortColumn == col:
			self.sortReverse = not self.sortReverse
		else:
			self.sortColumn = col
		self.update()
		
	def onListResize(self, event = None):
		colWidths = [ self.list.GetColumnWidth(col) for col in xrange(0,4) ]
		width = getListWidth(self.list) - colWidths[3]
		if width > 0:
			s = colWidths[0] + colWidths[1] + colWidths[2]
			w1 = int(width * (float(colWidths[0]) / s))
			w2 = int(width * (float(colWidths[1]) / s))
			self.list.SetColumnWidth(0, w1)
			self.list.SetColumnWidth(1, w2)
			self.list.SetColumnWidth(2, width - w1 - w2)
		if event: event.Skip()

	def onListConnect(self, event):
		getMainWnd().connectToHub(self.hubList[event.GetIndex()][1])
		
	def onListOptions(self, event):
		menu = wx.Menu()
		menu.Append(400, 'Connect')
		menu.Append(401, 'Add to favorites')
		
		self.list.SetItemState(event.GetIndex(), wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
		self.list.PopupMenu(menu, event.GetPoint())
	
	def onListPopup(self, event):
		id = event.GetId()
		if id == 401:
			settings = DCWorker.getWorker().getSettings()
		
		idx, first = -1, True
		for i in xrange(self.list.GetSelectedItemCount()):
			idx = self.list.GetNextItem(idx, state = wx.LIST_STATE_SELECTED)
			addr = self.hubList[idx][1]
			
			if id == 400:
				getMainWnd().connectToHub(addr, first)
				first = False
			elif id == 401:
				settings.addFavorite(addr, False, None, None)
		
		if id == 401:
			self.updateFavorites()

	def onManualConnect(self, event):
		addr = self.addr.GetValue()
		if len(addr) > 0:
			self.addr.SetValue('')
			getMainWnd().connectToHub(addr)
		else:
			idx, first = -1, True
			for i in xrange(self.list.GetSelectedItemCount()):
				idx = self.list.GetNextItem(idx, state = wx.LIST_STATE_SELECTED)
				getMainWnd().connectToHub(self.hubList[idx][1], first)
				first = False
	
	def onFavlistResize(self, event = None):
		width = getListWidth(self.favlist) - self.favlist.GetColumnWidth(1)
		if width > 0: self.favlist.SetColumnWidth(0, width)
		if event: event.Skip()
	
	def onFavlistSelection(self, event):
		self.favaddr.SetValue(self.favlist.GetItemText(event.GetIndex()))
		
	def onFavlistEdit(self, event):
		# Double notification workaround
		if self.vetoing:
			return event.Veto()
		
		newaddr = event.GetText().strip()
		settings = DCWorker.getWorker().getSettings()
		
		try:
			self.vetoing = True
			if not self.checkAddr(newaddr, settings):
				return event.Veto()
		finally:
			self.vetoing = False
		
		idx = self.favlist.GetNextItem(-1, state = wx.LIST_STATE_SELECTED)
		oldaddr = self.favlist.GetItemText(idx)
		fav = settings.getFavorite(oldaddr)
		settings.updateFavorite(oldaddr, newaddr, fav[1])

	def onFavlistConnect(self, event):
		getMainWnd().connectToHub(self.favlist.GetItemText(event.GetIndex()))
		
	def onFavlistOptions(self, event):
		menu = wx.Menu()
		menu.Append(500, 'Connect')
		menu.Append(501, 'Remove')
		menu.Append(502, 'Toggle autoconnect')
		
		self.favlist.SetItemState(event.GetIndex(), wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
		self.favlist.PopupMenu(menu, event.GetPoint())
	
	def onFavlistPopup(self, event):
		id = event.GetId()
		if id == 501:
			return self.onFavRemove()
		elif id == 502:
			settings = DCWorker.getWorker().getSettings()
		
		idx, first = -1, True
		for i in xrange(self.favlist.GetSelectedItemCount()):
			idx = self.favlist.GetNextItem(idx, state = wx.LIST_STATE_SELECTED)
			addr = self.favlist.GetItemText(idx)
			
			if id == 500:
				getMainWnd().connectToHub(addr, first)
				first = False
			elif id == 502:
				fav = settings.getFavorite(addr)
				settings.updateFavorite(addr, fav[0], not fav[1], fav[2], fav[3])
		
		if id == 502:
			self.updateFavorites()

	def onFavAdd(self, event):
		addr = self.favaddr.GetValue().strip()
		settings = DCWorker.getWorker().getSettings()
		
		if not self.checkAddr(addr, settings):
			return
		
		settings.addFavorite(addr, False, None, None)
		self.favaddr.SetValue('')
		self.updateFavorites()
	
	def onFavRemove(self, event = None):
		idx, lastidx = [], -1
		for i in xrange(self.favlist.GetSelectedItemCount()):
			lastidx = self.favlist.GetNextItem(lastidx, state = wx.LIST_STATE_SELECTED)
			idx.append(lastidx)
		
		idx.reverse()
		settings = DCWorker.getWorker().getSettings()
		for i in idx:
			settings.removeFavorite(self.favlist.GetItemText(i))
			self.favlist.DeleteItem(i)
	
	def onFavEdit(self, event):
		idx = self.favlist.GetNextItem(-1, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
		if idx == -1:
			return showErrorMessage(self, 'Please select a favorite.')
		
		settings = DCWorker.getWorker().getSettings()
		if FavoriteDialog(self, settings.getFavorite(self.favlist.GetItemText(idx))).ShowModal() == wx.ID_OK:
			self.updateFavorites()
	
	def onHublist(self, list):
		if list == None or len(list) == 0:
			if len(self.hubList) == 0:
				wx.PostEvent(self, wx.PyEvent(2))
		else:
			sys.modules['__main__'].hublist = list
			wx.PostEvent(self, wx.PyEvent(1))

	def update(self, event = None):
		self.hubList = sys.modules['__main__'].hublist
		self.list.DeleteAllItems()
		if len(self.hubList) == 0:
			return

		filter, reverse = self.sortAlgos[self.sortColumn]
		if self.sortReverse: reverse = not reverse
		
		if filter != None:
			aux = [ (filter(i[self.sortColumn]), i) for i in self.hubList ]
		else:
			aux = [ (i[self.sortColumn], i) for i in self.hubList ]

		aux.sort()
		if reverse: aux.reverse()
		self.hubList = [ i for (key, i) in aux ]
		
		self.list.Freeze()
		for pos, item in enumerate(self.hubList):
			self.list.InsertStringItem(pos, item[0])
			self.list.SetStringItem(pos, 1, item[1])
			self.list.SetStringItem(pos, 2, item[2])
			self.list.SetStringItem(pos, 3, str(item[3]))
		self.list.Thaw()
		self.onListResize()

	def displayError(self, event):
		self.list.DeleteAllItems()
		self.list.InsertStringItem(0, 'Error...')
	
	def updateFavorites(self):
		aux = [ (f[0].lower(), f[0], f[1]) for f in DCWorker.getWorker().getSettings().getFavorites() ]
		aux.sort()
		
		self.favlist.Freeze()
		self.favlist.DeleteAllItems()
		for pos, f in enumerate(aux):
			self.favlist.InsertStringItem(pos, f[1])
			if f[2]:
				self.favlist.SetStringItem(pos, 1, 'yes')
			else:
				self.favlist.SetStringItem(pos, 1, 'no')
		self.favlist.Thaw()

	#=================#
	# Private section #
	#=================#
	
	def checkAddr(self, addr, settings):
		if len(addr) == 0:
			showErrorMessage(self, 'You must specify an address.', 'Address error')
			return False
		
		if not settings.validateFavorite(addr, None, None):
			showErrorMessage(self, 'Invalid address.', 'Address error')
			return False
		
		if settings.getFavorite(addr):
			showErrorMessage(self, 'Address already in favorite list.', 'Address error')
			return False
		
		return True
	
	def itemFilter(self, data):
		return data.lower()
