# This file is part of pyDC software
# Copyright 2002-2004 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from ViewerListener import *
from Settings import *
from HublistPanel import *
from ViewsPanel import *
from HubsPanel import *
from QueuePanel import *
from XfersPanel import *
from SearchesPanel import *
from EmptyViewer import *
from HubViewer import *
from ChatViewer import *
from SearchViewer import *
from FileListViewer import *
from time import sleep
from Util import showErrorMessage, guiEvent
import wx
import wx.py

from DCHublistRetriever import *
from DCHub import *
from DCSearch import *
from DCFileList import *
import DCWorker

class MainWnd(wx.Frame, ViewerListener):
	CHAT_ENABLED = True
	LOAD_HUBLIST = False
	TITLE = 'pyDC'
	TITLE_PREFIX = { HubViewer : 'Hub:', ChatViewer : 'Chat:', SearchViewer : 'Search:', FileListViewer : 'Files of:' }
	VIEWER_INFO  = [ (DCHub, HubViewerInfo), (ChatJob, ChatViewerInfo), (DCSearch, SearchViewerInfo),  (DCUserList, FileListViewerInfo) ]
	VIEWER_CLASS = { EmptyViewerInfo: EmptyViewer, HubViewerInfo : HubViewer, ChatViewerInfo : ChatViewer, SearchViewerInfo : SearchViewer, FileListViewerInfo : FileListViewer }
	
	DEFAULT_AWAY_MESSAGE = "I'm away. Please leave a message."

	def __init__(self, parent, id, title):
		wx.Frame.__init__(self, parent, -1, title, size = (640, 480), style = wx.DEFAULT_FRAME_STYLE)
		self.infos = JobInfos()
		self.oldHeight = 0
		self.away = False
		self.timer = wx.Timer(self, 1)

		# GUI elements
		menuFile = wx.Menu()
		menuFile.Append(1001, 'Settings')
		menuFile.Append(1000, 'Exit')
		
		self.menuPreferences = wx.Menu()
		self.menuPreferences.AppendCheckItem(1003, 'Away')
		self.menuPreferences.AppendCheckItem(1004, 'Safe mode')
		self.menuPreferences.AppendCheckItem(1005, 'Show timestamps')
		
		menuBar = wx.MenuBar()
		menuBar.Append(menuFile, 'File')
		menuBar.Append(self.menuPreferences, 'Preferences')
		self.SetMenuBar(menuBar)
		
		self.hSplit = wx.SplitterWindow(self, -1, style = wx.NO_3D | wx.SP_NOBORDER)
		self.vSplit = wx.SplitterWindow(self.hSplit, -1, style = wx.NO_3D | wx.SP_3D)
		self.vSplit.SetMinimumPaneSize(5)

		self.notebook = wx.Notebook(self.vSplit, 1)

		self.hublist = HublistPanel(self.notebook, -1)
		self.notebook.AddPage(self.hublist, 'Hublist')
		
		self.views = ViewsPanel(self.notebook, -1)
		self.notebook.AddPage(self.views, 'Views')

		self.hubs = HubsPanel(self.notebook, -1)
		self.notebook.AddPage(self.hubs, 'Hubs')

		self.queue = QueuePanel(self.notebook, -1)
		self.notebook.AddPage(self.queue, 'Queue')

		self.xfers = XfersPanel(self.notebook, -1)
		self.notebook.AddPage(self.xfers, 'Xfers')
		
		self.searches = SearchesPanel(self.notebook, -1)
		self.notebook.AddPage(self.searches, 'Searches')

		self.viewer = EmptyViewer(self.vSplit, -1)
		
		self.shell = wx.py.shell.Shell(self.hSplit, -1)
		self.shell.autoComplete = 0
		self.shell.interp.runsource('from InterUtils import *')

		self.vSplit.SplitVertically(self.notebook, self.viewer)
		self.hSplit.SplitHorizontally(self.vSplit, self.shell)

		# Event handlers
		wx.EVT_SIZE(self.hSplit, self.onSplitSize)
		wx.EVT_CLOSE(self, self.onClose)
		wx.EVT_TIMER(self, 1, self.onTimer)

		# Accelerators
		self.SetAcceleratorTable(wx.AcceleratorTable([ (wx.ACCEL_CTRL, ord('Q'), 1000),
		                                               (wx.ACCEL_CTRL, ord('S'), 1002) ]))
		wx.EVT_MENU(self, 1000, self.onClose)
		wx.EVT_MENU(self, 1001, self.showSettings)
		wx.EVT_MENU(self, 1002, self.newSearch)
		wx.EVT_MENU(self, 1003, self.onAway)
		wx.EVT_MENU(self, 1004, self.onSafeMode)
		wx.EVT_MENU(self, 1005, self.onTimestamps)
		
		# Final setup
		worker = DCWorker.getWorker()
		worker.registerListener(self.hubs)
		worker.getSearchWorker().registerListener(self.searches)
		if self.LOAD_HUBLIST:
			retriever = DCHublistRetriever()
			retriever.registerListener(self.hublist)
			worker.addJob(retriever)
		
		self.Show(True)
		self.hSplit.SetSashPosition(380, True)
		self.vSplit.SetSashPosition(350, True)
		
		self.timer.Start(3000)
		self.hublist.addr.SetFocus()

	def onSplitSize(self, sizeEvent):
		newHeight = sizeEvent.GetSize().GetHeight()
		pos = self.hSplit.GetSashPosition() + newHeight - self.oldHeight
		self.hSplit.SetSashPosition(pos)
		self.oldHeight = newHeight
		sizeEvent.Skip()

	def onClose(self, event):
		DCWorker.getWorker().deregisterAllListeners()
		self.Destroy()

	def onTimer(self, event):
		self.hubs.update()
		self.queue.update()
		self.xfers.update()

		self.viewer.update()
	
	def onViewerTitle(self, viewer, title):
		if viewer != self.viewer: return
		
		if title == None or len(title) == 0:
			self.SetTitle(self.TITLE)
		else:
			self.SetTitle('%s - %s %s' % (self.TITLE, self.TITLE_PREFIX[viewer.__class__], title))

	def onViewerClosed(self, viewer, jobStopped):
		if self.viewer != viewer:
			self.infos.remove(viewer.getInfo().getJob())
		else:
			self.infos.pop()
			self.viewJob()
	
	def init(self):
		self.hublist.updateFavorites()
		self.queue.initQueue()
	
	def showSettings(self, event):
		d = SettingsDialog(self, DCWorker.getWorker().getSettings())
		d.Center()
		if d.ShowModal() == wx.ID_OK:
			DCWorker.getWorker().getSettings().save('settings.xml')

	def addFavorite(self, address):
		DCWorker.getWorker().getSettings().addFavorite(address, False, None, None)
		self.hublist.updateFavorites()
	
	def setAway(self, away):
		self.away = away
		self.menuPreferences.Check(1003, away)
		
		if self.away:
			for c in [ j for j in self.infos.jobs() if isinstance(j, ChatJob) ]:
				c.onAway(self.away)
	
	def isAway(self):
		return self.away
	
	def setSafeMode(self, safe):
		DCWorker.getWorker().getSettings().setCustom('safeMode', bool(safe))
		self.menuPreferences.Check(1004, safe)
	
	def setShowTimestamps(self, show):
		DCWorker.getWorker().getSettings().setCustom('showTimestamps', bool(show))
		self.menuPreferences.Check(1005, show)
	
	def newSearch(self, dummy=None):
		search = DCSearch()
		self.viewJob(search)
		self.searches.onNewSearch(search)
	
	def onAway(self, event):
		self.setAway(event.IsChecked())
	
	def onSafeMode(self, event):
		self.setSafeMode(event.IsChecked())
	
	def onTimestamps(self, event):
		self.setShowTimestamps(event.IsChecked())
	
	def connectToHub(self, addr, show = True):
		worker = DCWorker.getWorker()
		
		fav = worker.getSettings().getFavorite(addr)
		if fav:
			nick = fav[2]
			password = fav[3]
		else:
			nick = None
			password = None
	
		try:
			hub = DCHub(addr, nick, password)
			if show: self.viewJob(hub)
			worker.addJob(hub)
		
		except InvalidAddress:
			showErrorMessage(self, 'Invalid hub address: \'%s\'.' % addr, 'Address error')
	
	def viewJob(self, job = None):
		if job == None:
			oldViewer = None
			job = self.infos.firstJob()
		else:
			oldViewer = self.viewer
			if job == oldViewer.getInfo().getJob():
				return
		
		if job == None:
			info = EmptyViewerInfo()
		else:
			info = self.infos.peek(job)
			if info == None:
				for baseClass, infoClass in self.VIEWER_INFO:
					if isinstance(job, baseClass):
						info = infoClass(job)
						break
				self.infos.push(info)
		
		self.setupViewer(info)
	
	def getActiveJob(self):
		return self.viewer.getInfo().getJob()

	def getChat(self, nick, hub):
		if not MainWnd.CHAT_ENABLED: return None

		chat = self.infos.searchChat(nick, hub.getAddress())
		if chat:
			return chat
		else:
			chat = ChatJob(nick, hub)
			self.viewJob(chat)
			return chat
	
	def enableQueueRefresh(self, enable):
		self.queue.enableRefresh(enable)

	#===================#
	#  Private members  #
	#===================#

	def setupViewer(self, info):
		if info.viewer == None:
			self.viewer = self.VIEWER_CLASS[info.__class__](self.vSplit, -1, info)
			info.viewer = self.viewer
			
			if isinstance(info, HubViewerInfo):
				self.viewer.registerListener(self.hubs)
			elif isinstance(info, SearchViewerInfo):
				self.viewer.registerListener(self.searches)
			
			if not isinstance(info, EmptyViewerInfo):
				self.views.onNewViewer(self.viewer)
	
			self.viewer.registerListener(self)
			self.viewer.registerListener(self.views)
		else:
			self.viewer = info.viewer
			self.viewer.Show()

		sashPos = self.vSplit.GetSashPosition()
		self.vSplit.Unsplit()
		self.vSplit.SplitVertically(self.notebook, self.viewer, sashPos)
		
		self.viewer.SetFocus()
		self.onViewerTitle(self.viewer, self.viewer.getTitle())

class JobInfos:
	def __init__(self):
		self.l = []
	
	def push(self, info):
		self.l.insert(0, (info.getJob(), info))

	def pop(self):
		if len(self.l) > 0:
			del self.l[0]

	def firstJob(self):
		if len(self.l) == 0:
			return None
		else:
			return self.l[0][0]
	
	def firstInfo(self):
		if len(self.l) == 0:
			return None
		else:
			return self.l[0][1]

	def peek(self, job):
		for pos, pair in enumerate(self.l):
			if pair[0] == job:
				del self.l[pos]
				self.l.insert(0, pair)
				return pair[1]
		return None
	
	def remove(self, job):
		self.l = [ i for i in self.l if i[0] != job ]
		
	def jobs(self):
		for i in self.l: yield i[0]
	
	def infos(self):
		for i in self.l: yield i[1]
	
	def searchChat(self, nick, address):
		for pair in self.l:
			if isinstance(pair[0], ChatJob) and pair[0].getNick() == nick and pair[0].getAddress() == address:
				return pair[0]
		
		return None
