# This file is part of pyDC software
# Copyright 2002-2004 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from os import unlink
from Util import getMainWnd, getListWidth, guiEvent, caseFilter, formatSize
from ViewerListener import *
import os.path
import wx

from DCXfer import *
from DCQueueItem import *
from DCQueueEventListener import *
from DCUserList import DCUserList
from DCXferListenerEventListener import *
import DCWorker

class QueuePanel(wx.Panel, DCQueueEventListener, DCXferListenerEventListener, ViewerListener):
	def __init__(self, parent, id):
		wx.Panel.__init__(self, parent, id)
		self.items = []
		self.refresh = True
		self.menuId = -1
		
		self.sortAlgos = [ (caseFilter, False), (None, False), (None, True), (None, True) ]
		self.sortColumn = 2
		self.sortReverse = False

		# Controls
		self.list = wx.ListCtrl(self, 1, style = wx.LC_REPORT | wx.LC_SINGLE_SEL | wx.LC_VRULES)
		clear = wx.Button(self, 2, 'Clear completed')

		self.list.InsertColumn(0, 'File')
		self.list.InsertColumn(1, 'Status')
		self.list.InsertColumn(2, 'Progress', wx.LIST_FORMAT_RIGHT)
		self.list.InsertColumn(3, 'Size', wx.LIST_FORMAT_RIGHT)

		# Layout
		vLayout = wx.BoxSizer(wx.VERTICAL)
		vLayout.Add(self.list, 1, wx.EXPAND | wx.BOTTOM, 5)
		vLayout.Add(clear, 0, wx.EXPAND)
		
		self.list.Freeze()
		self.list.InsertStringItem(0, '###')
		self.list.SetStringItem(0, 1, 'Completed')
		self.list.SetStringItem(0, 2, '100.0%')
		self.list.SetStringItem(0, 3, '999.99 MB')
		self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)
		self.list.SetColumnWidth(2, wx.LIST_AUTOSIZE)
		self.list.SetColumnWidth(3, wx.LIST_AUTOSIZE)
		self.list.DeleteItem(0)
		self.list.Thaw()

		self.SetSizer(vLayout)
		self.Layout()

		# Events
		wx.EVT_LIST_COL_CLICK(self, 1, self.setSortAlgo)
		wx.EVT_SIZE(self.list, self.onListResize)
		wx.EVT_LIST_ITEM_RIGHT_CLICK(self, 1, self.onItemOptions)
		wx.EVT_LIST_ITEM_ACTIVATED(self, 1, self.onRemoveItem)
		wx.EVT_BUTTON(self, 2, self.onClear)
		wx.EVT_MENU_RANGE(self.list, 500, 502, self.onPopupSelection)
		self.Connect(1, 1, wx.wxEVT_NULL, self.update)
		self.Connect(2, 2, wx.wxEVT_NULL, self.removeItem)
		self.Connect(3, 3, wx.wxEVT_NULL, self.showList)

		DCWorker.getWorker().getQueue().registerListener(self)

	def setSortAlgo(self, event):
		col = event.GetColumn()
		if self.sortColumn == col:
			self.sortReverse = not self.sortReverse
		else:
			self.sortColumn = col
		self.update()
	
	def enableRefresh(self, enable):
		if enable and not self.refresh:
			wx.PostEvent(self, wx.PyEvent(1))
		
		self.refresh = enable
	
	def onListResize(self, event):
		w = getListWidth(self.list) - sum([ self.list.GetColumnWidth(col) for col in range(1,4) ])
		self.list.SetColumnWidth(0, w)
		event.Skip()
		
	def onItemSelected(self, event):
		try:
			DCWorker.getWorker().getQueue().addItem(self.results[event.GetIndex()])
		except ItemAlreadyAddedError:
			showErrorMessage(self, 'Item already queued.')
		
	def onItemOptions(self, event):
		pnt = event.GetPoint()
		item = self.items[event.GetIndex()]
		self.list.SetItemState(event.GetIndex(), wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

		menu = wx.Menu()
		menu.Append(500, 'Remove')
		menu.Append(501, 'Remove completed')
		menu.Append(502, 'Remove all')
		self.list.PopupMenu(menu, pnt)

		if self.menuId == -1:
			return
		elif self.menuId == 500:
			self.onRemoveItem(item)
		elif self.menuId == 501:
			DCWorker.getWorker().getQueue().removeCompletedItems()
		elif self.menuId == 502:
			for item in DCWorker.getWorker().getQueue().getItems():
				self.onRemoveItem(item)
			
		self.menuId = -1

	def onNewItem(self, queue, item):
		if self.refresh:
			wx.PostEvent(self, guiEvent(1, item))
		else:
			#CHECK: potential race condition
			self.items.append(item)
	
	def onItemStatus(self, queue, item):
		if isinstance(item, DCUserList) and item.getStatus() == DCQueueItem.STATUS_COMPLETED:
			wx.PostEvent(self, guiEvent(3, item))
		else:
			wx.PostEvent(self, wx.PyEvent(1))

	def onItemRemoved(self, queue, item):
		wx.PostEvent(self, guiEvent(2, item))

	def onRemoveItem(self, event):
		queue = DCWorker.getWorker().getQueue()
		if isinstance(event, DCQueueItem):
			item = event
		else:
			item = self.items[event.GetIndex()]
		path = item.getPath()
		
		if DCWorker.getWorker().getSettings().getCustom('safeMode') and \
		   wx.MessageDialog(self, "Remove '%s' from the queue?" % os.path.basename(path), 'Item removal', wx.YES_NO | wx
.ICON_QUESTION).ShowModal() == wx.ID_NO:
			return
		
		if item.getStatus() != DCQueueItem.STATUS_COMPLETED:
			if os.path.isfile(path):
				if wx.MessageDialog(self, 'Do you want to cancel the partially downloaded file?', os.path.basename(path), wx.YES_NO | wx.ICON_QUESTION).ShowModal() == wx.ID_YES:
					for path in item.getBlockPaths():
						try: unlink(path)
						except: pass
		
		queue.removeItem(item)
		
	def onClear(self, event):
		DCWorker.getWorker().getQueue().removeCompletedItems()
		
	def onPopupSelection(self, event):
		self.menuId = event.GetId()
	
	def removeItem(self, event):
		pos = self.items.index(event.data)
		del self.items[pos]
		self.list.DeleteItem(pos)
		
	def showList(self, event):
		DCWorker.getWorker().getQueue().removeItem(event.data)
		getMainWnd().viewJob(event.data)

	def initQueue(self):
		self.items = DCWorker.getWorker().getQueue().getItems()
		self.update()
	
	def update(self, event=None):
		if event != None and isinstance(event, guiEvent):
			self.items.append(event.data)
		snaps = self.sort()
		
		self.list.Freeze()
		count = self.list.GetItemCount()
		for pos, snap in enumerate(snaps):
			self.items[pos] = snap[-1]
			
			if pos < count:
				self.list.SetStringItem(pos, 0, os.path.basename(snap[0]))
			else:
				self.list.InsertStringItem(pos, os.path.basename(snap[0]))

			self.list.SetStringItem(pos, 1, snap[1])
			self.list.SetStringItem(pos, 2, "%.1f%%" % snap[2])
			self.list.SetStringItem(pos, 3, formatSize(snap[3]))
		self.list.Thaw()

	#===================#
	#  Private members  #
	#===================#
	
	def sort(self):
		snaps = [ (self.getName(i), i.getStatusLabel(), i.getProgress(), i.getSize(), i) for i in self.items ]
		filter, reverse = self.sortAlgos[self.sortColumn]
		if self.sortReverse: reverse = not reverse
		
		if filter != None:
			aux = [ (filter(s[self.sortColumn]), s) for s in snaps ]
		else:
			aux = [ (s[self.sortColumn], s) for s in snaps ]
			
		aux.sort()
		if reverse: aux.reverse()
		return [ a[1] for a in aux ]
	
	def getName(self, item):
		if isinstance(item, DCUserList):
			return item.getUser().nick + "'s list"
		else:
			return item.getPath()
