# This file is part of pyDC software
# Copyright 2002 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from Util import getListWidth, getMainWnd, guiEvent
from ViewerListener import *
import wx

from DCSearch import *
from DCSearchWorkerEventListener import *
from DCSearchEventListener import *
import DCWorker

class SearchesPanel(wx.Panel, ViewerListener, DCSearchWorkerEventListener, DCSearchEventListener):
	def __init__(self, parent, id):
		wx.Panel.__init__(self, parent, id)
		self.searches = []
		
		self.sortAlgos = [ (None, False), (None, True) ]
		self.sortColumn = 0
		self.sortReverse = False
		
		self.timer = wx.Timer(self, 5)

		# Controls
		self.list = wx.ListCtrl(self, 1, style = wx.LC_REPORT | wx.LC_VRULES)
		self.new = wx.Button(self, 2, 'New')

		self.list.InsertColumn(0, 'Pattern')
		self.list.InsertColumn(1, 'Hits')
		self.list.Show(1)

		# Layout
		vLayout = wx.BoxSizer(wx.VERTICAL)
		vLayout.Add(self.list, 1, wx.EXPAND)
		vLayout.Add(self.new, 0, wx.EXPAND | wx.TOP, 5)
		
		self.list.Freeze()
		self.list.InsertStringItem(0, '###')
		self.list.SetStringItem(0, 1, '99999')
		self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)
		self.list.DeleteItem(0)
		self.list.Thaw()

		self.SetSizer(vLayout)
		self.Layout()

		# Events
		wx.EVT_LIST_COL_CLICK(self, 1, self.setSortAlgo)
		wx.EVT_SIZE(self.list, self.onListResize)
		wx.EVT_LIST_ITEM_SELECTED(self, 1, self.onSearchActivation)
		wx.EVT_BUTTON(self, 2, self.onSearch)
		self.Connect(1, 1, wx.wxEVT_NULL, self.addSearch)
		self.Connect(2, 2, wx.wxEVT_NULL, self.removeSearch)
		self.Connect(3, 3, wx.wxEVT_NULL, self.update)
		self.Connect(4, 4, wx.wxEVT_NULL, self.startTimer)
		wx.EVT_TIMER(self, 5, self.onTimer)

	def setSortAlgo(self, event):
		col = event.GetColumn()
		if self.sortColumn == col:
			self.sortReverse = not self.sortReverse
		else:
			self.sortColumn = col
		self.update()
		
	def onListResize(self, event):
		width = getListWidth(self.list) - self.list.GetColumnWidth(1)
		self.list.SetColumnWidth(0, width)
		event.Skip()

	def onSearchActivation(self, event):
		getMainWnd().viewJob(self.searches[event.GetIndex()])

	def onSearch(self, event):
		search = getMainWnd().newSearch()

	def onNewSearch(self, search):
		if search.isPrivate(): return
		self.AddPendingEvent(guiEvent(1, search))
		
	def onSearchStop(self, search):
		self.AddPendingEvent(guiEvent(2, search))

	def onSearchStart(self, search):
		self.AddPendingEvent(wx.PyEvent(3))

	def onNewResult(self, search, result):
		self.AddPendingEvent(wx.PyEvent(4))

	def onViewerClosed(self, viewer, stoppedJob):
		event = guiEvent(3, viewer.getInfo().getJob())
		self.removeSearch(event)

	def addSearch(self, event):
		search = event.data
		search.registerListener(self)
		
		try:
			self.searches.index(search)
 			return
		except ValueError:
			self.searches.append(search)

		self.update()
		
	def removeSearch(self, event):
		try:
			pos = self.searches.index(event.data)
			event.data.deregisterListener(self)
			del self.searches[pos]
			self.list.DeleteItem(pos)
		except ValueError:
			return
	
	def update(self, event=None):
		snaps = self.sort()
		
		self.list.Freeze()
		count = self.list.GetItemCount()
		for pos, snap in enumerate(snaps):
			self.searches[pos] = snap[-1]
			
			if pos < count:
				self.list.SetStringItem(pos, 0, snap[0])
			else:
				self.list.InsertStringItem(pos, snap[0])
			
			self.list.SetStringItem(pos, 1, str(snap[1]))
		self.list.Thaw()
	
	def startTimer(self, event):
		if not self.timer.IsRunning():
			self.timer.Start(1000, True)
	
	def onTimer(self, event):
		self.timer.Stop()
		self.update()

	#=================#
	# Private section #
	#=================#
	
	def patternFilter(self, data):
		if data == None: data = '[New search]'
		return data.lower()

	def sort(self):
		snaps = [ (self.patternFilter(s.getPattern()), s.getResultNum(), s) for s in self.searches ]
		filter, reverse = self.sortAlgos[self.sortColumn]
		if self.sortReverse: reverse = not reverse
		
		if filter != None:
			aux = [ (filter(s[self.sortColumn]), s) for s in snaps ]
		else:
			aux = [ (s[self.sortColumn], s) for s in snaps ]
			
		aux.sort()
		if reverse: aux.reverse()
		return [ a[1] for a in aux ]
