# This file is part of pyDC software
# Copyright 2002-2003 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from socket import error
from Util import showErrorMessage
import os.path
import wx

from DCUser import DCUser
from DCSettings import DCSettings
import DCWorker

class SettingsDialog(wx.Dialog):
	def __init__(self, parent, settings, refresh = True):
		wx.Dialog.__init__(self, parent, -1, 'pyDC settings', style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
		self._settings = settings
		self._refresh = refresh

		notebook = wx.Notebook(self, -1)
		ns = wx.NotebookSizer(notebook)
		notebook.SetSizer(ns)

		# User info tab
		userinfo = wx.Panel(notebook, -1)
		self.nick = wx.TextCtrl(userinfo, -1)
		self.connection = wx.Choice(userinfo, -1, choices = [ c[1] for c in DCUser.CONN_NAMES.items() if c[0] != DCUser.CONN_NONE ])
		self.description = wx.TextCtrl(userinfo, -1)
		self.email = wx.TextCtrl(userinfo, -1)

		gs1 = wx.FlexGridSizer(cols = 2, hgap = 5, vgap = 5)
		gs1.AddGrowableCol(1)
		gs1.Add(wx.StaticText(userinfo, -1, 'Nick:'), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
		gs1.Add(self.nick, 1, wx.EXPAND | wx.RIGHT, 5)
		gs1.Add(wx.StaticText(userinfo, -1, 'Connection:'), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
		gs1.Add(self.connection, 1, wx.EXPAND | wx.RIGHT, 5)
		gs1.Add(wx.StaticText(userinfo, -1, 'Description:'), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
		gs1.Add(self.description, 1, wx.EXPAND | wx.RIGHT, 5)
		gs1.Add(wx.StaticText(userinfo, -1, 'Email:'), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
		gs1.Add(self.email, 1, wx.EXPAND | wx.RIGHT, 5)

		userinfo.SetSizer(gs1)
		notebook.AddPage(userinfo, 'User info')

		# Network tab
		network = wx.Panel(notebook, -1)
		modeBox = wx.StaticBox(network, -1, 'Mode')
		slotBox = wx.StaticBox(network, -1, 'Slots')
		bandBox = wx.StaticBox(network, -1, 'Bandwidth limiters (0 = no limit)')
		listBox = wx.StaticBox(network, -1, 'Hublists')

		# Modes box
		self.activeButton = wx.RadioButton(network, 1, 'Active', style = wx.RB_GROUP)
		self.forceIp = wx.CheckBox(network, 2, 'Force IP:')
		self.ip = wx.TextCtrl(network, -1)
		self.port = wx.TextCtrl(network, -1)
		self.passiveButton = wx.RadioButton(network, 3, 'Passive')

		gs2 = wx.FlexGridSizer(cols = 2, hgap = 5, vgap = 5)
		gs2.AddGrowableCol(1)
		gs2.Add(self.forceIp, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		gs2.Add(self.ip, 1, wx.EXPAND | wx.LEFT, 5)
		gs2.Add(wx.StaticText(network, -1, 'Port:'), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		gs2.Add(self.port, 1, wx.EXPAND | wx.LEFT, 5)

		bs3 = wx.StaticBoxSizer(modeBox, wx.VERTICAL)
		bs3.Add(self.activeButton, 0, wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)
		bs3.Add(gs2, 0, wx.EXPAND | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)
		bs3.Add(self.passiveButton, 0, wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)
		# END modes box

		# Slots box
		self.slots = wx.SpinCtrl(network, -1)
		self.slots.SetRange(1, 99)

		bs4 = wx.StaticBoxSizer(slotBox, wx.HORIZONTAL)
		bs4.Add(wx.StaticText(network, -1, 'Upload slots:'), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		bs4.Add(self.slots, 1, wx.EXPAND | wx.RIGHT, 5)
		# END slots box
		
		# Bandwidth box
		self.uploadBand = wx.TextCtrl(network, -1)
		self.downloadBand = wx.TextCtrl(network, -1)
		
		gs3 = wx.FlexGridSizer(cols = 3, hgap = 5, vgap = 5)
		gs3.AddGrowableCol(1)
		
		gs3.Add(wx.StaticText(network, -1, 'Upload limit:'), 0, wx.ALIGN_CENTER_VERTICAL)
		gs3.Add(self.uploadBand, 1, wx.EXPAND)
		gs3.Add(wx.StaticText(network, -1, 'KBps'), 0)
		
		gs3.Add(wx.StaticText(network, -1, 'Download limit:'), 0, wx.ALIGN_CENTER_VERTICAL)
		gs3.Add(self.downloadBand, 1, wx.EXPAND)
		gs3.Add(wx.StaticText(network, -1, 'KBps'), 0)
		
		bs5 = wx.StaticBoxSizer(bandBox, wx.VERTICAL)
		bs5.Add(gs3, 1, wx.EXPAND | wx.ALL, 5)
		# END bandwidth box

		# Hublists box
		self.hublists = wx.ListBox(network, 4)
		self.listaddr = wx.TextCtrl(network, -1)
		self.listremove = wx.Button(network, 5, 'Remove')

		gs4 = wx.FlexGridSizer(cols = 2, hgap = 5, vgap = 5)
		gs4.AddGrowableRow(0)
		gs4.AddGrowableCol(0)

		gs4.Add(self.hublists, 1, wx.EXPAND)
		gs4.Add(self.listremove, 0)
		gs4.Add(self.listaddr, 1, wx.EXPAND)
		gs4.Add(wx.Button(network, 6, 'Add'), 0)

		bs6 = wx.StaticBoxSizer(listBox, wx.VERTICAL)
		bs6.Add(gs4, 1, wx.EXPAND | wx.ALL, 5)
		# END hublists box

		bs7 = wx.BoxSizer(wx.VERTICAL)
		bs7.Add(bs3, 0, wx.EXPAND | wx.ALL, 5)
		bs7.Add(bs4, 0, wx.EXPAND | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)
		bs7.Add(bs5, 0, wx.EXPAND | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)
		bs7.Add(bs6, 1, wx.EXPAND | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)

		network.SetSizer(bs7)
		notebook.AddPage(network, 'Network')

		# Share tab
		share = wx.Panel(notebook, -1)
		shareBox = wx.StaticBox(share, -1, 'Shared directories')
		downloadBox = wx.StaticBox(share, -1, 'Downloads')

		# Shares box
		self.updlist = wx.ListBox(share, 7)
		self.updpath = wx.TextCtrl(share, -1)
		self.updremove = wx.Button(share, 9, 'Remove')

		bs8 = wx.BoxSizer(wx.VERTICAL)
		bs8.Add(wx.Button(share, 8, 'Add'), 0, wx.BOTTOM, 5)
		bs8.Add(self.updremove, 0)

		gs5 = wx.FlexGridSizer(cols = 2, hgap = 5, vgap = 5)
		gs5.AddGrowableRow(0)
		gs5.AddGrowableCol(0)

		gs5.Add(self.updlist, 1, wx.EXPAND | wx.RIGHT, 5)
		gs5.Add(bs8, 0)
		gs5.Add(self.updpath, 1, wx.EXPAND | wx.RIGHT, 5)
		gs5.Add(wx.Button(share, 10, 'Browse'), 0)

		bs9 = wx.StaticBoxSizer(shareBox, wx.VERTICAL)
		bs9.Add(gs5, 1, wx.EXPAND | wx.ALL, 5)
		# END shares box

		# Downloads box
		self.downpath = wx.TextCtrl(share, -1)

		bs10 = wx.BoxSizer(wx.HORIZONTAL)
		bs10.Add(wx.StaticText(share, -1, 'Download directory:'), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
		bs10.Add(self.downpath, 1, wx.EXPAND | wx.RIGHT, 5)
		bs10.Add(wx.Button(share, 11, 'Browse'), 0, wx.RIGHT, 5)

		bs11 = wx.StaticBoxSizer(downloadBox, wx.VERTICAL)
		bs11.Add(bs10, 0, wx.EXPAND)
		# END downloads box

		bs12 = wx.BoxSizer(wx.VERTICAL)
		bs12.Add(bs9, 1, wx.EXPAND | wx.ALL, 5)
		bs12.Add(bs11, 0, wx.EXPAND | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)

		share.SetSizer(bs12)
		notebook.AddPage(share, 'Share')

		# Button panel
		bs13 = wx.BoxSizer(wx.HORIZONTAL)
		bs13.Add(wx.Button(self, wx.ID_OK, 'OK'), 0, wx.RIGHT, 10)
		bs13.Add(wx.Button(self, wx.ID_CANCEL, 'Cancel'))

		# Final setup
		bs14 = wx.BoxSizer(wx.VERTICAL)
		bs14.Add(notebook, 1, wx.EXPAND)
		bs14.Add(bs13, 0, wx.ALIGN_RIGHT | wx.TOP | wx.BOTTOM | wx.RIGHT, 5)
		self.SetSizer(bs14)
		self.SetSize(wx.Size(500, 480))

		self.listremove.Enable(0)
		self.updremove.Enable(0)

		# Load values
		if settings.user.nick != None: self.nick.SetValue(settings.user.nick)
		if settings.user.connection != DCUser.CONN_NONE:
			self.connection.SetStringSelection(DCUser.CONN_NAMES[settings.user.connection])
		if settings.user.description != None: self.description.SetValue(settings.user.description)
		if settings.user.email != None: self.email.SetValue(settings.user.email)

		if settings.active:
			self.activeButton.SetValue(1)
			self.passiveButton.SetValue(0)
		else:
			self.activeButton.SetValue(0)
			self.passiveButton.SetValue(1)
			self.forceIp.Enable(0)
			self.ip.Enable(0)
			self.port.Enable(0)

		if settings.ip != None:
			self.forceIp.SetValue(1)
			self.ip.SetValue(settings.ip)
		else:
			self.ip.Enable(0)

		if settings.port <= 0:
			self.port.SetValue(str(settings.DEFAULT_PORT))
		else:
			self.port.SetValue(str(settings.port))
		
		if settings.slotNum < 1:
			self.slots.SetValue(settings.DEFAULT_SLOTNUM)
		else:
			self.slots.SetValue(settings.slotNum)
		
		self.uploadBand.SetValue(str(settings.uploadBandwidth / 1024.0))
		self.downloadBand.SetValue(str(settings.downloadBandwidth / 1024.0))

		if len(settings.hubLists) > 0:
			for source in settings.hubLists:
				self.hublists.Append(source)
		else:
			self.hublists.Append(settings.DEFAULT_HUBLIST)

		if len(settings.sharedDirs) > 0:
			for d in settings.sharedDirs:
				self.updlist.Append(d)

		if settings.downloadDir != None: self.downpath.SetValue(settings.downloadDir)

		# Setup events
		wx.EVT_BUTTON(self, wx.ID_OK, self.onOK)
		wx.EVT_RADIOBUTTON(self, 1, self.onActive)
		wx.EVT_CHECKBOX(self, 2, self.onForceIp)
		wx.EVT_RADIOBUTTON(self, 3, self.onPassive)
		wx.EVT_LISTBOX(self, 4, self.onListSelection)
		wx.EVT_BUTTON(self, 5, self.onListRemove)
		wx.EVT_BUTTON(self, 6, self.onListAdd)
		wx.EVT_LISTBOX(self, 7, self.onUpdSelection)
		wx.EVT_BUTTON(self, 8, self.onUpdAdd)
		wx.EVT_BUTTON(self, 9, self.onUpdRemove)
		wx.EVT_BUTTON(self, 10, self.onBrowseUpd)
		wx.EVT_BUTTON(self, 11, self.onBrowseDown)

	def onOK(self, event):
		try:
			nick = self.nick.GetValue()
			if len(nick) == 0 or not DCSettings.validateNick(nick):
				raise ValueError, 'invalid nick'

			try:
				connName = self.connection.GetStringSelection()
				connection = [ i[0] for i in DCUser.CONN_NAMES.items() if i[1] == connName ][0]
			except IndexError:
				raise ValueError, 'invalid connection'

			description = self.description.GetValue()
			if not DCSettings.validateDescription(description):
				raise ValueError, 'invalid description'

			email = self.email.GetValue()
			if len(email) != 0 and not DCSettings.validateEmail(email):
				raise ValueError, 'invalid email address'

			active = self.activeButton.GetValue()
			ip = None
			if active and self.forceIp.GetValue():
				ip = self.ip.GetValue()
				if not self.checkIp(ip):
					raise ValueError, 'invalid IP address'
			
			try:
				port = int(self.port.GetValue())
				if not DCSettings.validatePort(port): raise ValueError
			except ValueError:
				raise ValueError, 'invalid port number'

			slots = int(self.slots.GetValue())
			
			try:
				uploadBandwidth = float(self.uploadBand.GetValue()) * 1024
				if not DCSettings.validateUploadBandwidth(uploadBandwidth): raise ValueError
			except ValueError:
				raise ValueError, 'invalid upload bandwidth limit'
			
			try:
				downloadBandwidth = float(self.downloadBand.GetValue()) * 1024
				if not DCSettings.validateDownloadBandwidth(downloadBandwidth): raise ValueError
			except ValueError:
				raise ValueError, 'invalid download bandwidth limit'

			hubLists = []
			for i in range(self.hublists.GetCount()):
				url = self.hublists.GetString(i)
				if not DCSettings.validateHubList(url):
					raise ValueError, 'invalid hublist address "%s"' % url
				hubLists.append(url)
			if len(hubLists) == 0:
				raise ValueError, 'no hublist specified'

			updPaths = []
			for i in range(self.updlist.GetCount()):
				path = self.updlist.GetString(i)
				if not DCSettings.validateSharedDir(path, updPaths):
					raise ValueError, 'invalid shared dir "%s"' % path
				updPaths.append(path)
			if len(updPaths) == 0:
				raise ValueError, 'no shared directory specified'

			downPath = self.downpath.GetValue()
			if not DCSettings.validateDownloadDir(downPath):
				raise ValueError, 'invalid download path'
			
			self._settings.user.nick = nick
			self._settings.user.connection = connection
			self._settings.user.description = description
			self._settings.user.email = email
			self._settings.active = active
			self._settings.ip = ip
			self._settings.port = port
			self._settings.slotNum = slots
			self._settings.uploadBandwidth = uploadBandwidth
			self._settings.downloadBandwidth = downloadBandwidth
			self._settings.hubLists = hubLists
			self._settings.sharedDirs = updPaths
			self._settings.downloadDir = downPath

			self.EndModal(wx.ID_OK)
			
			try:
				if self._refresh: DCWorker.getWorker().refreshSettings()
			except error:
				msg  = 'Error while binding socket to port %d.\nEither another application is '  % DCWorker.getWorker().getSettings().port
				msg += 'using it or you have insufficient permissions.\nChange port number setting'
				msg += ' or you won\'t be able to start searches and file transfers.'
				showErrorMessage(self, msg, 'Socket error')
			
		except ValueError, e:
			showErrorMessage(self, 'Error: %s.' % str(e), 'Settings error')

	def onForceIp(self, event):
		self.ip.Enable(self.forceIp.GetValue())

	def onActive(self, event):
		self.forceIp.Enable(1)
		self.ip.Enable(self.forceIp.GetValue())
		self.port.Enable(1)

	def onPassive(self, event):
		self.forceIp.Enable(0)
		self.ip.Enable(0)
		self.port.Enable(0)

	def onListSelection(self, event):
		self.listremove.Enable(1)
		self.listaddr.SetValue(self.hublists.GetString(self.hublists.GetSelection()))

	def onListRemove(self, event):
		self.hublists.Delete(self.hublists.GetSelection())
		self.listremove.Enable(0)

	def onListAdd(self, event):
		addr = self.listaddr.GetValue()

		try:
			if len(addr) == 0:
				raise ValueError, 'empty address'
			elif len(addr) < 7 or addr[:7] != 'http://':
				raise ValueError, 'address has no \'http://\' prefix'
			
			self.hublists.Append(addr)
			self.listaddr.SetValue('')
		
		except ValueError, msg:
			showErrorMessage(self, 'Error: %s.' % msg, 'List address error')

	def onUpdSelection(self, event):
		self.updremove.Enable(1)
		self.updpath.SetValue(self.updlist.GetString(self.updlist.GetSelection()))

	def onUpdAdd(self, event):
		path = self.updpath.GetValue()

		try:
			if len(path) == 0:
				raise ValueError, 'empty path'
			elif not os.path.isdir(path):
				raise ValueError, '\'%s\' does not exist' % path

			self.updlist.Append(path)
			self.updpath.SetValue('')
		
		except ValueError, msg:
			showErrorMessage(self, 'Error: %s.' % msg, 'Share directory error')

	def onUpdRemove(self, event):
		self.updlist.Delete(self.updlist.GetSelection())
		self.updremove.Enable(0)

	def onBrowseUpd(self, event):
		d = wx.DirDialog(self)
		d.ShowModal()
		path = d.GetPath()
		if len(path) > 0:
			self.updpath.SetValue(path)

	def onBrowseDown(self, event):
		d = wx.DirDialog(self)
		d.ShowModal()
		path = d.GetPath()
		if len(path) > 0:
			self.downpath.SetValue(path)

	def checkIp(self, value):
		tokens = value.split('.')
		if len(tokens) != 4: return False

		try:
			for token in tokens: int(token)
		except ValueError:
			return False

		return True
