# This file is part of pyDC software
# Copyright 2002-2003 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

import sys
import wx

##
## Functions
##
def getMainWnd():
	return sys.modules["__main__"].wnd

def getListWidth(list):
	# NOTE: on GTK, the scrollbar is included in the client size, but on
	# Windows it is not included
	width = list.GetClientSize().GetWidth()
	if wx.Platform != '__WXMSW__' and list.GetCountPerPage() > 0 and list.GetItemCount() > list.GetCountPerPage():
		width -= wx.SystemSettings_GetSystemMetric(wx.SYS_VSCROLL_X)

	return width

def showErrorMessage(parent, message, title = 'Error'):
	wx.MessageDialog(parent, message, title, wx.OK | wx.ICON_ERROR).ShowModal()

def boolFilter(data):
	return not (data.lower() in ('0', 'false', 'no'))

def caseFilter(data):
	return data.lower()

def basename(path):
	try:
		return path[path.rindex('\\')+1:]
	except ValueError:
		return path

def formatSize(size):
	try:
		if size < 1024: return '%d b' % size

		size /= 1024.0
		if size < 1024: return '%.2f kb' % size

		size /= 1024
		if size < 1024: return '%.2f MB' % size

		size /= 1024
		if size < 1024: return '%.2f GB' % size

		size /= 1024
		return '%.2f TB' % size
	except OverflowError:
		return 'too large'
	
def formatSpeed(speed):
	if speed < 1024: return '%d b/s' % speed
	
	speed /= 1024.0
	if speed < 1024: return '%.2f kb/s' % speed

	speed /= 1024
	return '%.2f MB/s' % speed

def formatEta(secs):
	part = secs % 3600
	h = (secs - part) / 3600
	if h > 9999: return '00:00:00'

	secs = part

	part = secs % 60
	m = (secs - part) / 60
	secs = part

	return "%.2d:%.2d:%.2d" % (h, m, secs)

##
## Classes
##
class guiEvent(wx.PyEvent):
	def __init__(self, id, data):
		wx.PyEvent.__init__(self, id)
		self.data = data

