# This file is part of pyDC software
# Copyright 2002-2003 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from ChatViewer import ChatJob
from Util import getMainWnd, getListWidth, guiEvent
from ViewerListener import *
import wx

from DCHub import DCHub
from DCSearch import DCSearch
from DCUserList import DCUserList

class ViewsPanel(wx.Panel, ViewerListener):
	JOB_LABELS = [ (DCHub, 'Hub'), (ChatJob, 'Chat'), (DCSearch, 'Search'), (DCUserList, 'File list') ]
	
	def __init__(self, parent, id):
		wx.Panel.__init__(self, parent, id)
		self.items = []

		# Controls
		self.list = wx.ListCtrl(self, 1, style = wx.LC_REPORT | wx.LC_SINGLE_SEL | wx.LC_VRULES)
		self.list.InsertColumn(0, 'Type')
		self.list.InsertColumn(1, 'Name')

		# Layout
		vLayout = wx.BoxSizer(wx.VERTICAL)
		vLayout.Add(self.list, 1, wx.EXPAND | wx.TOP, 5)
		
		self.list.Freeze()
		self.list.InsertStringItem(0, 'Search')
		self.list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
		self.list.DeleteItem(0)
		self.list.Thaw()
		
		self.SetSizer(vLayout)
		self.Layout()

		# Events
		wx.EVT_SIZE(self.list, self.onListResize)
		wx.EVT_LIST_ITEM_SELECTED(self.list, 1, self.onViewerActivation)

	def onListResize(self, event):
		width = getListWidth(self.list) - self.list.GetColumnWidth(0)
		self.list.SetColumnWidth(1, width)
		event.Skip()

	def onViewerActivation(self, event):
		getMainWnd().viewJob(self.items[event.GetIndex()][-1])

	def onNewViewer(self, viewer):
		job = viewer.getInfo().getJob()
		self.items.append([self.jobLabel(job), viewer.getTitle(), job])
		self.items.sort()
		idx = self.jobIndex(job)
		
		self.list.Freeze()
		self.list.InsertStringItem(idx, self.items[idx][0])
		self.list.SetStringItem(idx, 1, self.items[idx][1])
		self.list.Thaw()

	def onViewerTitle(self, viewer, title):
		job = viewer.getInfo().getJob()
		self.items[self.jobIndex(job)][1] = title
		self.items.sort()
		
		self.list.Freeze()
		for pos, item in enumerate(self.items):
			self.list.SetStringItem(pos, 0, item[0])
			self.list.SetStringItem(pos, 1, item[1])
		self.list.Thaw()

	def onViewerClosed(self, viewer, jobStopped):
		idx = self.jobIndex(viewer.getInfo().getJob())
		del self.items[idx]
		self.list.DeleteItem(idx)

	#=================#
	# Private section #
	#=================#

	def jobLabel(self, job):
		for baseClass, label in self.JOB_LABELS:
			if isinstance(job, baseClass):
				return label
		return 'Unknown'
		
	def jobIndex(self, job):
		for pos, item in enumerate(self.items):
			if item[-1] == job:
				return pos
		return -1
