# This file is part of pyDC software
# Copyright 2002-2004 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from time import time
from Util import guiEvent, getListWidth, caseFilter, formatSpeed, formatEta
from ViewerListener import *
import os.path
import wx

from DCQueueItem import DCQueueItem
from DCXfer import *
from DCXferListenerEventListener import *
import DCWorker

class XfersPanel(wx.Panel, ViewerListener, DCXferListenerEventListener):
	INFO_TIMEOUT = 3

	def __init__(self, parent, id):
		wx.Panel.__init__(self, parent, id)
		self.upds = []
		self.downs = []
		
		self.sortAlgos = [ (caseFilter, False), (caseFilter, False), (None, True), (None, True), (None, False) ]
		self.downColumn = 2
		self.downReverse = False
		self.updColumn = 2
		self.updReverse = False
		self.menuId = -1
		
		# Controls
		downloads = wx.StaticBox(self, -1, 'Downloads')
		uploads = wx.StaticBox(self, -1, 'Uploads')

		# Downloads box
		self.downList = wx.ListCtrl(self, 1, style = wx.LC_REPORT | wx.LC_VRULES)
		self.downList.InsertColumn(0, 'User')
		self.downList.InsertColumn(1, 'File')
		self.downList.InsertColumn(2, 'Progress', wx.LIST_FORMAT_RIGHT)
		self.downList.InsertColumn(3, 'Speed', wx.LIST_FORMAT_RIGHT)
		self.downList.InsertColumn(4, 'ETA', wx.LIST_FORMAT_RIGHT)

		bs1 = wx.StaticBoxSizer(downloads, wx.VERTICAL)
		bs1.Add(self.downList, 1, wx.EXPAND)
		
		self.downList.Freeze()
		self.downList.InsertStringItem(0, '###')
		self.downList.SetStringItem(0, 2, '100.0%')
		self.downList.SetStringItem(0, 3, '999.9 MB/s')
		self.downList.SetStringItem(0, 4, '99:99:99')
		self.downList.SetColumnWidth(2, wx.LIST_AUTOSIZE)
		self.downList.SetColumnWidth(3, wx.LIST_AUTOSIZE)
		self.downList.SetColumnWidth(4, wx.LIST_AUTOSIZE)
		self.downList.DeleteItem(0)
		self.downList.Thaw()
		# END downloads box

		# Uploads box
		self.updList = wx.ListCtrl(self, 2, style = wx.LC_REPORT | wx.LC_VRULES)
		self.updList.InsertColumn(0, 'User')
		self.updList.InsertColumn(1, 'File')
		self.updList.InsertColumn(2, 'Progress', wx.LIST_FORMAT_RIGHT)
		self.updList.InsertColumn(3, 'Speed', wx.LIST_FORMAT_RIGHT)
		self.updList.InsertColumn(4, 'ETA', wx.LIST_FORMAT_RIGHT)

		bs2 = wx.StaticBoxSizer(uploads, wx.VERTICAL)
		bs2.Add(self.updList, 1, wx.EXPAND)
		
		self.updList.Freeze()
		self.updList.InsertStringItem(0, '###')
		self.updList.SetStringItem(0, 2, '100.0%')
		self.updList.SetStringItem(0, 3, '999.9 MB/s')
		self.updList.SetStringItem(0, 4, '99:99:99')
		self.updList.SetColumnWidth(2, wx.LIST_AUTOSIZE)
		self.updList.SetColumnWidth(3, wx.LIST_AUTOSIZE)
		self.updList.SetColumnWidth(4, wx.LIST_AUTOSIZE)
		self.updList.DeleteItem(0)
		self.updList.Thaw()
		# END uploads box

		# Final setup
		vLayout = wx.BoxSizer(wx.VERTICAL)
		vLayout.Add(bs1, 3, wx.EXPAND)
		vLayout.Add(bs2, 2, wx.EXPAND)

		self.SetSizer(vLayout)
		self.Layout()

		# Setup events
		wx.EVT_LIST_COL_CLICK(self, 1, self.setDownAlgo)
		wx.EVT_SIZE(self.downList, self.onDownListResize)
		wx.EVT_LIST_ITEM_RIGHT_CLICK(self, 1, self.onDownOptions)
		wx.EVT_LIST_ITEM_ACTIVATED(self.downList, 1, self.closeDown)
		wx.EVT_MENU_RANGE(self.downList, 500, 502, self.onPopupSelection)
		wx.EVT_LIST_COL_CLICK(self, 2, self.setUpdAlgo)
		wx.EVT_SIZE(self.updList, self.onUpdListResize)
		wx.EVT_LIST_ITEM_ACTIVATED(self.updList, 2, self.closeUpd)
		self.Connect(1, 1, wx.wxEVT_NULL, self.addXfer)
		self.Connect(2, 2, wx.wxEVT_NULL, self.removeXfer)
		
		DCWorker.getWorker().getXferWorker().registerListener(self)
		
	def setDownAlgo(self, event):
		col = event.GetColumn()
		if self.downColumn == col:
			self.downReverse = not self.downReverse
		else:
			self.downColumn = col
		self.update(1)
	
	def onDownListResize(self, event):
		self.resizeList(self.downList)
		event.Skip()
	
	def onDownOptions(self, event):
		try: down = self.downs[event.GetIndex()]
		except IndexError: return

		self.downList.SetItemState(event.GetIndex(), wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

		menu = wx.Menu()
		menu.Append(500, 'Stop')
		menu.Append(501, 'Copy nick')
		menu.Append(502, 'Remove user from queue')

		self.downList.PopupMenu(menu, event.GetPoint())

		if self.menuId == 500:
			down.stop()
			
		elif self.menuId == 501:
			clipdata = wx.TextDataObject() 
			clipdata.SetText(down.getRemoteNick())
			wx.TheClipboard.Open() 
			wx.TheClipboard.SetData(clipdata) 
			wx.TheClipboard.Close()
		
		elif self.menuId == 502:
			for item in DCWorker.getWorker().getQueue().getItems():
				item.removeSource(down.getRemoteNick())
		
		self.menuId = -1
	
	def closeDown(self, event):
		down = self.downs[event.GetIndex()]
		
		if DCWorker.getWorker().getSettings().getCustom('safeMode') and \
		   wx.MessageDialog(self, "Stop '%s' download?" % os.path.basename(down.getItem().getPath()), 'Download', wx.YES_NO | wx.ICON_QUESTION).ShowModal() == wx.ID_NO:
			return
		
		down.stop()
	
	def onPopupSelection(self, event):
		self.menuId = event.GetId()

	def setUpdAlgo(self, event):
		col = event.GetColumn()
		if self.updColumn == col:
			self.updReverse = not self.updReverse
		else:
			self.updColumn = col
		self.update(2)
		
	def onUpdListResize(self, event):
		self.resizeList(self.updList)
		event.Skip()
	
	def closeUpd(self, event):
		up = self.upds[event.GetIndex()]
		
		if DCWorker.getWorker().getSettings().getCustom('safeMode') and \
		   wx.MessageDialog(self, "Stop '%s' upload?" % os.path.basename(up.getItem().getPath()), 'Upload', wx.YES_NO | wx.ICON_QUESTION).ShowModal() == wx.ID_NO:
			return
		
		up.stop()
	
	def onNewXfer(self, xfer):
		wx.PostEvent(self, guiEvent(1, xfer))
		
	def onXferClosed(self, xfer):
		wx.PostEvent(self, guiEvent(2, xfer))
	
	def addXfer(self, event):
		xfer = event.data
		
		if xfer.getType() == DCXfer.TYPE_DOWNLOAD:
			self.downs.append(xfer)
			self.update(1)
		else:
			self.upds.append(xfer)
			self.update(2)
	
	def removeXfer(self, event):
		xfer = event.data
		
		if xfer.getType() == DCXfer.TYPE_DOWNLOAD:
			pos = self.downs.index(xfer)
			del self.downs[pos]
			self.downList.DeleteItem(pos)
		else:
			pos = self.upds.index(xfer)
			del self.upds[pos]
			self.updList.DeleteItem(pos)

	def update(self, mask = 3):
		if mask & 1 == 1:
			self.updateList(self.downs, self.downList, self.downColumn, self.downReverse)
		
		if mask & 2 == 2:
			self.updateList(self.upds, self.updList, self.updColumn, self.updReverse)
	
	#=================#
	# Private section #
	#=================#
	
	def resizeList(self, list):
		colWidths = [ list.GetColumnWidth(col) for col in range(0,5) ]
		width = getListWidth(list) - sum(colWidths[2:])
		if width > 0:
			scaleFactor = float(colWidths[0]) / (colWidths[0] + colWidths[1])
			w = int(width * scaleFactor)
			list.SetColumnWidth(0, w)
			list.SetColumnWidth(1, width - w)
	
	def updateList(self, xfers, list, sortColumn, sortReverse):
		snap = self.sort(xfers, sortColumn, sortReverse)
		
		list.Freeze()
		count = list.GetItemCount()
		for pos, s in enumerate(snap):
			xfers[pos] = s[-1]
			
			if pos < count:
				list.SetStringItem(pos, 0, s[0])
			else:
				list.InsertStringItem(pos, s[0])
			
			self.updateFields(list, pos, s)
		
		list.Thaw()
		
	def sort(self, xfers, sortColumn, sortReverse):
		snap = [ (x.getRemoteNick(), self.xferPath(x), x.getProgress(), x.getSpeed(), self.xferEta(x), x) for x in xfers ]
		filter, reverse = self.sortAlgos[sortColumn]
		if sortReverse: reverse = not reverse
		
		if filter != None:
			aux = [ (filter(s[sortColumn]), s) for s in snap ]
		else:
			aux = [ (s[sortColumn], s) for s in snap ]
			
		aux.sort()
		if reverse: aux.reverse()
		return [ a[1] for a in aux ]

	def updateFields(self, list, pos, snap):
		list.SetStringItem(pos, 1, snap[1])
		list.SetStringItem(pos, 2, '%.1f%%' % snap[2])
		list.SetStringItem(pos, 3, formatSpeed(snap[3]))
		list.SetStringItem(pos, 4, formatEta(snap[4]))
		
	def xferPath(self, xfer):
		item = xfer.getItem()
		if item:
			path = os.path.basename(item.getPath())
			if xfer.isCompressed():
				path = '*' + path
			if isinstance(item, DCQueueItem) and item.hasManyBlocks():
				path = '!' + path
			return path
		else:
			return ''

	def xferEta(self, xfer):
		speed = xfer.getSpeed()
		if speed > 0:
			secs = xfer.getSizeRemaining() / speed
		else:
			secs = 0

		return secs
