# This file is part of pyDC software
# Copyright 2003-2004 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from Channel import *

class DCChannel(Channel):
	def __init__(self):
		Channel.__init__(self)
		self.cmdBuf = ''
		self.debug = False #DEBUG

	def recvCmd(self):
		self.cmdBuf += self.sock.recv()
		bufLen = len(self.cmdBuf)
		
		while bufLen > 0:
			pos = self.cmdBuf.find('|')
			if pos == -1: break
			elif pos > 0:
				yield DCChannelCommand(self.cmdBuf[:pos])

			pos +=1
			self.cmdBuf = self.cmdBuf[pos:]
			bufLen -= pos
	
	def unrecvCmd(self):
		self.sock.unrecv(self.cmdBuf)
		self.cmdBuf = ''

	def sendCmd(self, cmd, *args):
		cmdstr = '$' + cmd
		for i in args:
			cmdstr += ' ' + i
		cmdstr += '|'

		if self.debug:
			print "[%s::sendCmd] >> %s" % (self.__class__, cmdstr)
		if self.sock:
			self.sock.send(cmdstr)
	
	def onDisconnection(self):
		self.cmdBuf = ''

class DCChannelHandler(ChannelHandler):
	def __init__(self, commands):
		ChannelHandler.__init__(self)
		self.commands = commands
	
	def poll(self):
		try:
			for cmd in self.channel.recvCmd():
				if self.channel.handler != self:
					self.channel.unrecvCmd()
					raise StopIteration
			
				if self.channel.debug:
					if cmd.name != 'Search':
						print '[%s::poll] <<' % self, cmd
				
				if cmd.name == None:
					self.onChatMessage(cmd.data)
					continue
				
				try:
					prototype = self.commands[cmd.name]
				except KeyError:
					self.onUnknownCommand(cmd)
					continue
				
				try:
					if prototype[0] > 0:
						args = cmd.data.split(' ')
						if len(args) == prototype[0]:
							prototype[1](self, *args)
						else:
							self.onInvalidCommand(cmd)

					elif prototype[0] == 0:
						if cmd.data == None:
							prototype[1](self)
						else:
							self.onInvalidCommand(cmd)
					
					else:
						prototype[1](self, cmd.data)
				
				except InvalidCommand:
					self.onInvalidCommand(cmd)
		
		except StopIteration:
			return
		
		if self.channel.handler != self:
			self.channel.unrecvCmd()
	
	def onChatMessage(self, block):
		pass

	def onInvalidCommand(self, block):
		pass

	def onUnknownCommand(self, block):
		pass

class DCChannelCommand:
	def __init__(self, block):
		isNamed = block[0] == '$'

		if isNamed:
			pos = block.find(' ')
			if pos == -1:
				self.name = block[1:]
				self.data = None
			else:
				self.name = block[1:pos]
				self.data = block[pos+1:]
		else:
			self.name = None
			self.data = block
	
	def __str__(self):
		return 'Command: %s, %s' % (self.name, self.data)

class InvalidCommand(Exception): pass
