# This file is part of pyDC software
# Copyright 2002-2003 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from threading import Lock
from weakref import ref

class EventGenerator:
	def __init__(self):
		self.lock = Lock()
		self.listeners = { None : [] }

	def registerListener(self, listener, interface = None):
		try:
			self.lock.acquire()
			
			if self.findRef(listener, interface): return
			self.listeners[interface].append(ref(listener, self.refFin))
		finally:
			self.lock.release()

	def deregisterListener(self, listener, interface = None):
		try:
			self.lock.acquire()
			
			r = self.findRef(listener, interface)
			if r: self.listeners[interface].remove(r)
		finally:
			self.lock.release()

	def deregisterAllListeners(self):
		self.lock.acquire()
		for key in self.listeners.iterkeys():
			self.listeners[key] = []
		self.lock.release()

	def getListeners(self, interface = None):
		# Thread unsafe; caller must acquire self.lock
		return [ l() for l in self.listeners[interface] ]

	#=================#
	# Private members #
	#=================#

	def addInterface(self, interface, default = True):
		self.lock.acquire()
		self.listeners[interface] = []
		if default:
			self.listeners[None] = self.listeners[interface]
		self.lock.release()
		
	def findRef(self, listener, interface):
		# Thread unsafe; caller must acquire self.lock
		for l in self.listeners[interface]:
			if l() == listener: return l
		return None
	
	def refFin(self, ref):
		self.lock.acquire()
		for key in self.listeners.iterkeys():
			try: self.listeners[key].remove(ref)
			except ValueError: pass
		self.lock.release()
