# This file is part of pyDC software
# Copyright 2003 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from copy import copy
from threading import Lock
from time import time

class JobPool:
	def __init__(self):
		self.jobs = []
		self.addedJobs = None
		self.removedJobs = None
		self.lock = Lock()

	def addJob(self, job):
		self.lock.acquire()
		if self.addedJobs == None:
			self.addedJobs = []
		self.addedJobs.append(job)
		self.lock.release()

	def removeJob(self, job):
		self.lock.acquire()
		if self.removedJobs == None:
			self.removedJobs = []
		self.removedJobs.append(job)
		self.lock.release()

	def run(self):
		self.lock.acquire()

		# Update job list
		if self.addedJobs != None:
			for job in self.addedJobs:
				if not job in self.jobs:
					self.jobs.append(job)
			self.addedJobs = None

		if self.removedJobs != None:
			for job in self.removedJobs:
				try: self.jobs.remove(job)
				except: pass
			self.removedJobs = None

		self.lock.release()
		self.jobs = [ j for j in self.jobs if not j.poll() ]

	def getJobs(self):
		self.lock.acquire()
		jobs = copy(self.jobs)
		self.lock.release()
		return jobs

class ClosePool:
	TIMEOUT = 5

	def __init__(self):
		self.jobs = []
		self.addedJobs = None
		self.lock = Lock()

	def addJob(self, job):
		job.stop()

		self.lock.acquire()
		if self.addedJobs == None:
			self.addedJobs = []
		self.addedJobs.append([job, time()])
		self.lock.release()

	def hasJobs(self):
		return len(self.jobs) > 0

	def run(self):
		if self.addedJobs != None:
			self.lock.acquire()
			for item in self.addedJobs:
				self.jobs.append(item)
			self.addedJobs = None
			self.lock.release()

		t = time()
		self.jobs = [ j for j in self.jobs if not j[0].poll() and t - j[1] < self.TIMEOUT ]
