# This file is part of pyDC software
# Copyright 2004 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

from array import array
from struct import unpack
import md5

class BloomSummary:
	LOAD_FACTOR = 8
	
	def __init__(self, items):
		self.bits = items * self.LOAD_FACTOR
		self.bitmap = array('B', '\0' * (self.bits >> 3))
	
	def add(self, pattern):
		m = md5.new()
		m.update(pattern)
		
		idxs = unpack('4I', m.digest())
		for i in idxs:
			i %= self.bits
			offset, mask = i >> 3, 1 << (i % 8)
			self.bitmap[offset] |= mask
	
	def match(self, pattern):
		m = md5.new()
		m.update(pattern)
		
		idxs = unpack('4I', m.digest())
		for i in idxs:
			i %= self.bits
			offset, mask = i >> 3, 1 << (i % 8)
			if self.bitmap[offset] & mask != mask:
				return False
		
		return True

if __name__ == '__main__':
	import os
	import sys
	
	if len(sys.argv) != 2:
		sys.stderr.write('%s PATH' % sys.argv[0])
		sys.exit(1)
	
	count = 0
	names = []
	for root, dirs, files in os.walk(sys.argv[1]):
		count += len(files)
		names += files
	
	b = BloomSummary(count)
	for name in names:
		aux = []
		s = ''
		
		for c in name:
			if c.isalnum():
				s += c
			else:
				if len(s) > 0:
					aux.append(s)
					s = ''
		if len(s): aux.append(s)
		
		for pattern in aux:
			b.add(pattern)
	
	while True:
		print "Query: ",
		query = sys.stdin.readline()
		if len(query) == 0: break
		
		found = True
		for p in [ p for p in query[:-1].split(' ') if len(p) ]:
			if not b.match(p):
				found = False
				continue
		
		if found:
			print "There is something matching your query."
		else:
			print "No result found."
