/*This file is part of pyDC software
Copyright 2002 Anakim Border <aborder@users.sourceforge.net>

pyDC is released under the terms of GPL licence.*/

#include "Python.h"
#include <iostream>
#include <list>

using namespace std;

class BitStream
{
public:
	BitStream(unsigned char* data, unsigned int size = static_cast<unsigned int>(-1));
	unsigned int getBit();
	unsigned int getBits(unsigned int num);
	void setBit(const unsigned char value);
	void setBits(const unsigned int value, const int len);
	void pad();
	unsigned int getByteNum() const;

protected:
	unsigned char *m_data, *m_end;
	unsigned int m_pos, m_num;
};

class Encoder
{
public:
	Encoder() : m_encoded(NULL), m_len(0) {}
	~Encoder() { delete m_encoded; }
	void encode(const char* data);
	void encoded(char **ptr, int *len) { *ptr = m_encoded;  *len = m_len; }
	const unsigned int length() const { return m_len; }

protected:
	typedef struct PatternInfo
	{
		PatternInfo() : value(0), pattern(0), len(0), count(0) {}

		unsigned char value;
		unsigned int pattern;
		unsigned char len;

		unsigned int count;
	};

	typedef struct TreeNode
	{
		TreeNode(PatternInfo *info) : weight(info->count), l(NULL), r(NULL), v(info) {}
		TreeNode(TreeNode* left, TreeNode *right) : weight(left->weight + right->weight), l(left), r(right), v(NULL) {}
		~TreeNode() { delete l; delete r; }
		void setPattern(unsigned int base = 0, unsigned char len = 0);

		unsigned int weight;
		TreeNode *l;
		TreeNode *r;
		PatternInfo *v;
	};

	void insertNode(list<TreeNode*> &l, TreeNode &node);
	//void print_tree(list<TreeNode*> &nodes) const;

	char* m_encoded;
	unsigned int m_len;
};

class Decoder
{
public:
	Decoder() : m_decoded(NULL) {}
	~Decoder() { delete m_decoded; }
	bool decode(char* data, unsigned int size);
	const char* decoded() { return m_decoded; }

protected:
	typedef struct PatternInfo
	{
		unsigned char value;
		unsigned char length;
	};

	char* m_decoded;
};

class IndexError
{
};

class CompressionError
{
};
