# Copyright 2003 Anakim Border <aborder@users.sourceforge.net>
# Based on the work of Wolfgang Strobl
#
# Original copyright notice
#
# (c) 2001 Copyright by Wolfgang Strobl ws@mystrobl.de,
#          License analog to the current Python license

import string, re
import _winreg

def getNameservers():
	local = _winreg.ConnectRegistry(None, _winreg.HKEY_LOCAL_MACHINE)
	res = _scanRegistry(local)
	_winreg.CloseKey(local)
	return res
	
def _scanRegistry(local):
	# Windows ME
	server = _getValue(local, r"SYSTEM\CurrentControlSet\Services\VxD\MSTCP", 'NameServer')
	if server != None:
		nameservers = [ _stringdisplay(server) ]
		return
	
	# Windows NT
	server = _getValue(local, r"SYSTEM\CurrentControlSet\Services\Tcpip\Parameters", 'NameServer')
	if server != None and len(server) > 0:
		nameservers = [ server[0] ]
		return
		
	# Windows 2000
	servers = _getEnumValues(local, r"SYSTEM\CurrentControlSet\Services\Tcpip\Parameters\DNSRegisteredAdapters", 'DNSServerAddresses')
	if len(servers):
		nameservers = [ _binipdisplay(s) for s in servers ]
		return
	
	# Windows XP
	nameservers = []
	servers = _getEnumValues(local, r"SYSTEM\CurrentControlSet\Services\Tcpip\Parameters\Interfaces", 'NameServer')
	if len(servers):
		for s in servers:
			nameservers += _stringdisplay(s)
		return nameservers
	
	servers = _getEnumValues(local, r"SYSTEM\CurrentControlSet\Services\Tcpip\Parameters\Interfaces", 'DhcpNameServer')
	for s in servers:
		nameservers += _stringdisplay(s)
	return nameserver

def _getValue(local, key, label):
	try:
		k = _winreg.OpenKey(local, key)
		value, type = _winreg.QueryValueEx(k, label)
		_winreg.CloseKey(k)
		return value
	except EnvironmentError:
		return None

def _getEnumValues(local, key, label):
	res = []
	
	try:
		k = _winreg.OpenKey(local, key)
		
		for i in range(1000):
			n = _winreg.EnumKey(k, i)
			sk = _winreg.OpenKey(k, n)
			
			try:
				value, type = _winreg.QueryValueEx(sk, label)
				res.append(value)
			except:
				pass
		
		_winreg.CloseKey(sk)
	except:
		_winreg.CloseKey(k)
	
	return res
	
def _stringdisplay(s):
	"""convert "d.d.d.d,d.d.d.d" to ["d.d.d.d","d.d.d.d"];
	also handle u'd.d.d.d d.d.d.d', as reporting on SF"""
	return map(str, re.split("[ ,]",s))

def _binipdisplay(s):
	"""convert a binary array of ip adresses to a python list"""
	if len(s) % 4 != 0: return []
	
	ol=[]
	for i in range(len(s)/4):
		s1=s[:4]
		s=s[4:]
		ip=[]
		for j in s1:
			ip.append(str(ord(j)))
			ol.append(string.join(ip,'.'))
	return ol

if __name__ == '__main__':
    print getNameservers()
