#!/usr/bin/env python
# This file is part of pyDC software
# Copyright 2002-2004 Anakim Border <aborder@users.sourceforge.net>
#
# pyDC is released under the terms of GPL licence.

# Fix import path
from os import sep
import sys

sys.path.insert(0, 'pyDClib' + sep + 'he3')
sys.path.insert(0, 'pyDClib')
sys.path.insert(0, 'inter')

from MainWnd import *
from Settings import *
from os.path import isfile
from socket import error
from Util import boolFilter, showErrorMessage
import wx

from DCLocalIP import LocalIPError
from DCSettings import *
from DCHublistRetriever import *
from DCQueueItem import IllegalItem
import DCWorker

wnd = None

class App(wx.App):
	def OnInit(self):
		global wnd
		
		worker = DCWorker.getWorker()
		sys.modules['__main__'].hublist = []
		
		wnd = MainWnd(None, -1, "pyDC")
		self.SetTopWindow(wnd)
		
		try: worker.loadQueue()
		except:
			showErrorMessage(wnd, "The file 'queue.pk' is corrupted. Please remove it and restart pyDC.", 'Queue error')
			wnd.Close()
			return True
		
		if not isfile('settings.xml'):
			if not self.showSettings(worker):
				wnd.Close()
				return True
		else:
			try:
				try:
					busy = wx.BusyInfo('Loading settings...')
					worker.loadSettings('settings.xml')
				finally:
					del busy
			
			except ParseError, e:
				showErrorMessage(wnd, str(e), 'Configuration error')
				if not self.showSettings(worker):
					wnd.Close()
					return True
				
			except LocalIPError:
				showErrorMessage(wnd, 'Error while detecting local IP.\nPlease provide it manually.', 'IP detection')
				if not self.showSettings(worker, False):
					wnd.Close()
					return True
				
				if worker.getSettings().getIP() == None:
					showErrorMessage(wnd, 'pyDC won\'t start if not properly configured.', 'Settings error')
					wnd.Close()
					return True
			
			except error:
				msg  = 'Error while binding socket to port %d.\nEither another application is '  % worker.getSettings().port
				msg += 'using it or you have insufficient permissions.\nChange port number setting'
				msg += ' or you won\'t be able to start searches and file transfers.'
				showErrorMessage(wnd, msg, 'Socket error')


		s = worker.getSettings()
		s.getCustomEx('awayMessage', default=wnd.DEFAULT_AWAY_MESSAGE)
		wnd.setShowTimestamps(s.getCustomEx('showTimestamps', boolFilter, False))
		wnd.setSafeMode(s.getCustomEx('safeMode', boolFilter, False))
			
		wnd.init()
		worker.start()

		return True
		
	def showSettings(self, worker, refresh = True):
		d = SettingsDialog(wnd, worker.getSettings(), refresh)
		d.Center()
		if d.ShowModal() != wx.ID_OK:
			showErrorMessage(wnd, 'pyDC won\'t start if not properly configured.', 'Settings error')
			return False
		else:
			worker.getSettings().save('settings.xml')
			return True

if __name__ == "__main__":
	print "+----------------------------------------------------------+"
	print "| pyDC                                                     |"
	print "| Direct Connect client written in Python.                 |"
	print "|                                                          |"
	print "| Written by Anakim Border <aborder@users.sourceforge.net> |"
	print "| Version 0.7                                              |"
	print "+----------------------------------------------------------+"
	print

	app = App(True)
	app.MainLoop()
	wx.TheClipboard.Clear() #prevents segfault on Linux
	
	worker = DCWorker.getWorker()
	if worker.isAlive():
		worker.saveSettings('settings.xml')
		worker.stop()
